// PropsDlg.cpp: implementation of the CPropsDlg class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PropsDlg.h"
#include "resource.h"
#include <Commdlg.h>
#include <shlwapi.h>
#include "tlbhotkeyctl.h"
#include "commctrl.h"
#include ".\propsdlg.h"
#include "treetabctl.h"
#include <Mlang.h>
#include "weblinkctl.h"
#include "shlobj.h"
#include "cmdlineBtn.h"

extern SEARCH_ENGINE defaultEngines[];

void Version2Str(DWORD versionMS, DWORD versionLS, LPTSTR str)
{
	DWORD ver[4];
	ver[0] = HIWORD(versionMS);
	ver[1] = LOWORD(versionMS);
	ver[2] = HIWORD(versionLS);
	ver[3] = LOWORD(versionLS);
	int lastNum = 3;
	for(int i=3; i >=0; i--)
	{
		if(ver[i])
		{
			lastNum = i;
			break;
		}
	}
	if(lastNum < 1) lastNum = 1;
	str[0] = 0;
	for(int i=0; i <= lastNum; i++)
	{
		TCHAR num[100];
		itoa(ver[i], num, 10);
		lstrcat(str, num);
		if(i != lastNum)
		{
			lstrcat(str, TEXT("."));
		}
	}
}

void GetFileVersion(LPCTSTR fileName, DWORD& versionMS, DWORD& versionLS)
{
	versionMS = 0;
	versionLS = 0;
	DWORD dw;
	DWORD verSize = GetFileVersionInfoSize((LPTSTR) fileName, &dw);
	if(verSize)
	{
		void* verData = malloc(verSize);
		if(GetFileVersionInfo((LPTSTR) fileName, NULL, verSize, verData))
		{
			VS_FIXEDFILEINFO* verInfo;
			UINT len = 0;
			if(VerQueryValue(verData, TEXT("\\"), (void**) &verInfo, &len))
			{
				versionMS = verInfo->dwFileVersionMS;
				versionLS = verInfo->dwFileVersionLS;
			}
		}
		free(verData);
	}
}


void GetHKString(WORD hk, LPTSTR str)
{
	if(!hk)
	{
		lstrcpy(str, TEXT("None"));
		return;
	}
	str[0] = 0;
	//Ctrl
	if(hk & 0x0200)
	{
		lstrcat(str, TEXT("Ctrl + "));
	}
	//Shift
	if(hk & 0x0100)
	{
		lstrcat(str, TEXT("Shift + "));
	}
	//Alt
	if(hk & 0x0400)
	{
		lstrcat(str, TEXT("Alt + "));
	}
	//Win
	if(hk & 0x2000)
	{
		lstrcat(str, TEXT("Win + "));
	}
	TCHAR keyName[255];
	keyName[0] = 0;
	UINT scanCode = MapVirtualKey(hk & 0xFF, 0);
	if(hk & 0x0800)
	{
		scanCode |= 0x00000100;
	}
	if(!GetKeyNameText(scanCode << 16, keyName, 255))
	{
		if(hk & 0xFF)
		{
			wsprintf(keyName, TEXT("#%d"), hk & 0xFF);
		}
	}
	lstrcat(str, keyName);
}


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPropsDlg::CPropsDlg() : CTrueDialog(IDD_PROPS)
{
	m_font = NULL;
}

CPropsDlg::~CPropsDlg()
{
	if(m_font) DeleteObject(m_font);
}

int CALLBACK MyEnumFonts(ENUMLOGFONTEX *lpelfe, NEWTEXTMETRICEX *lpntme, DWORD FontType, LPARAM lParam)
{
	HWND hwndCB = (HWND) lParam;
	if(SendMessage(hwndCB, CB_FINDSTRING, NULL, (LPARAM) lpelfe->elfFullName) == CB_ERR)
	{
		SendMessage(hwndCB, CB_ADDSTRING, NULL, (LPARAM) lpelfe->elfFullName);
	}
	return TRUE;
}

void CPropsDlg::OnInitDialog()
{
	HDC hdc = GetDC(m_hWnd);
	LOGFONT lf;
	ZeroMemory(&lf, sizeof(lf));
	lf.lfCharSet = DEFAULT_CHARSET;
	EnumFontFamiliesEx(hdc, &lf, (FONTENUMPROC) MyEnumFonts, (LPARAM) GetDlgItem(m_hWnd, IDC_FONTNAME), 0);
	ReleaseDC(m_hWnd, hdc);

	SetDlgItemInt(m_hWnd, IDC_SZ, sz, FALSE);
	SetDlgItemInt(m_hWnd, IDC_HISTORYSIZE, m_historySize, FALSE);

	SendDlgItemMessage(m_hWnd, IDC_HOTKEY, HKM_SETHOTKEY, m_hotKey, 0);
	SendDlgItemMessage(m_hWnd, IDC_HIDE, BM_SETCHECK, m_autoHide, 0);

	SendDlgItemMessage(m_hWnd, IDC_ENABLEAUTOSUGGESTLIST, BM_SETCHECK, m_enableASList, 0);
	SendDlgItemMessage(m_hWnd, IDC_ENABLEAUTOAPPEND, BM_SETCHECK, m_enableAUTOAPPEND, 0);

	SetDlgItemInt(m_hWnd, IDC_FONTSIZE, m_fontSize, FALSE);
	SetDlgItemText(m_hWnd, IDC_FONTNAME, m_fontName);
	SendDlgItemMessage(m_hWnd, IDC_BOLD, BM_SETCHECK, m_fontBold, 0);
	SendDlgItemMessage(m_hWnd, IDC_ITALIC, BM_SETCHECK, m_fontItalic, 0);

	CreateCmdFont();

	FindSkins();
	InitSkin();
}

DWORD CPropsDlg::OnNotify(int idCtrl, LPNMHDR pnmh)
{
	switch(pnmh->code)
	{
	case PSN_APPLY:
		{
			sz = GetDlgItemInt(m_hWnd, IDC_SZ, NULL, FALSE);
			m_historySize = GetDlgItemInt(m_hWnd, IDC_HISTORYSIZE, NULL, FALSE);
			m_hotKey = (WORD) SendDlgItemMessage(m_hWnd, IDC_HOTKEY, HKM_GETHOTKEY, 0, 0);
			m_autoHide = SendDlgItemMessage(m_hWnd, IDC_HIDE, BM_GETCHECK, 0, 0);

			m_enableASList = SendDlgItemMessage(m_hWnd, IDC_ENABLEAUTOSUGGESTLIST, BM_GETCHECK, 0, 0);
			m_enableAUTOAPPEND = SendDlgItemMessage(m_hWnd, IDC_ENABLEAUTOAPPEND, BM_GETCHECK, 0, 0);

			m_fontSize = GetDlgItemInt(m_hWnd, IDC_FONTSIZE, NULL, FALSE);
			int idx = SendDlgItemMessage(m_hWnd, IDC_FONTNAME, CB_GETCURSEL, 0, 0);
			if(idx < 0)
			{
				GetDlgItemText(m_hWnd, IDC_FONTNAME, m_fontName, 32);
			} else
			{
				SendDlgItemMessage(m_hWnd, IDC_FONTNAME, CB_GETLBTEXT, idx, (LPARAM) m_fontName);
			}
			m_fontBold = SendDlgItemMessage(m_hWnd, IDC_BOLD, BM_GETCHECK, 0, 0);
			m_fontItalic = SendDlgItemMessage(m_hWnd, IDC_ITALIC, BM_GETCHECK, 0, 0);

			m_skinPath[0] = 0;
			int skinIDX = GetCBData(IDC_SKIN);
			if(skinIDX >=0 && skinIDX < m_skins.GetCount())
			{
				GetRelSkinPath(m_skins[skinIDX].path, m_skinPath);
			}
		}
		return PSNRET_NOERROR;
	}
	return 0;
}



void CPropsDlg::OnCommand(DWORD itemID)
{
	switch(LOWORD(itemID))
	{
	case IDC_SKIN:
		if(HIWORD(itemID) == CBN_SELCHANGE)
		{
			InitSkin();
			RedrawPreview();
		}
		break;
	case IDC_BOLD:
	case IDC_ITALIC:
		if(HIWORD(itemID) == BN_CLICKED)
		{
			CreateCmdFont();
		}
		break;
	case IDC_FONTSIZE:
		if(HIWORD(itemID) == EN_CHANGE)
		{
			CreateCmdFont();
		}
		break;
	case IDC_FONTNAME:
		if( HIWORD(itemID) == EN_CHANGE || 
			HIWORD(itemID) == CBN_EDITCHANGE ||
			HIWORD(itemID) == CBN_SELCHANGE)
		{
			CreateCmdFont();
		}
		break;
	case IDC_SELECTFILE:
		{
			TCHAR fileName[MAX_PATH];
			fileName[0] = 0;
			OPENFILENAME ofn;
			memset(&ofn, 0, sizeof(ofn));
			ofn.lStructSize = sizeof(ofn);
			ofn.hwndOwner = m_hWnd;
			ofn.hInstance = g_hInst;
			ofn.lpstrFilter = TEXT("PNG files\0*.png\0");
			ofn.lpstrCustomFilter = NULL;
			ofn.lpstrFile = fileName;
			ofn.nMaxFile = MAX_PATH;
			ofn.lpstrTitle = "Select picture for spacer";
			ofn.Flags = OFN_ENABLESIZING | OFN_FILEMUSTEXIST | OFN_LONGNAMES | OFN_PATHMUSTEXIST;
			if(GetOpenFileName(&ofn))
			{
				SetDlgItemText(m_hWnd, IDC_FILENAME, fileName);
				SetVis();
			}
		}
		break;
	}
}

void CPropsDlg::SetVis(void)
{
/*	char txt[MAX_PATH];
	GetDlgItemText(m_hWnd, IDC_FILENAME, txt, MAX_PATH);
	if(PathFileExists(txt))
	{
		EnableWindow(GetDlgItem(m_hWnd, IDC_SZ), FALSE);
		EnableWindow(GetDlgItem(m_hWnd, IDC_ALIGN), TRUE);
	} else
	{
		EnableWindow(GetDlgItem(m_hWnd, IDC_SZ), TRUE);
		EnableWindow(GetDlgItem(m_hWnd, IDC_ALIGN), FALSE);
	}*/
}

CCmdProperties::CCmdProperties(void) : CTrueDialog(IDD_PROPERTIES)
{
	m_tab = 0;
}

CCmdProperties::~CCmdProperties(void)
{
}


void CCmdProperties::OnInitDialog(void)
{
	TTAB_PAGE items[5];

	char strGeneral[255];
	char strGeneralL[255];
	LoadString(m_hResInst, IDS_GENERAL, strGeneral, 255);
	LoadString(m_hResInst, IDS_L_GENERALOPTIONS, strGeneralL, 255);
	items[0].hInstance		= g_hInst;
	items[0].imgID			= IDR_GENERAL;
	items[0].pageID			= PAGE_GENERAL;
	items[0].pfnDlgProc		= (DLGPROC) CTrueDialog::WndProc;
	items[0].lParam			= (LPARAM) &m_pageGeneral;
	items[0].pResource		= m_pageGeneral.GetTemplate(m_hResInst);
	items[0].pszTitle		= strGeneral;
	items[0].pszPageLabel	= strGeneralL;

	char strHistory[255];
	char strHistoryL[255];
	LoadString(m_hResInst, IDS_HISTORY, strHistory, 255);
	LoadString(m_hResInst, IDS_L_HISTORY, strHistoryL, 255);
	items[1].hInstance	= g_hInst;
	items[1].imgID		= IDR_HISTRY;
	items[1].pageID		= PAGE_HISTORY;
	items[1].pfnDlgProc = (DLGPROC) CTrueDialog::WndProc;
	items[1].lParam		= (LPARAM) &m_pageHistory;
	items[1].pResource	= m_pageHistory.GetTemplate(m_hResInst);
	items[1].pszTitle	= strHistory;
	items[1].pszPageLabel	= strHistoryL;

	char strAlias[255];
	char strAliasL[255];
	LoadString(m_hResInst, IDS_ALIASES, strAlias, 255);
	LoadString(m_hResInst, IDS_ALIASES, strAliasL, 255);
	items[2].hInstance	= g_hInst;
	items[2].imgID		= IDR_ALIAS;
	items[2].pageID		= PAGE_ALIAS;
	items[2].pfnDlgProc = (DLGPROC) CTrueDialog::WndProc;
	items[2].lParam		= (LPARAM) &m_pageAlias;
	items[2].pResource	= m_pageAlias.GetTemplate(m_hResInst);
	items[2].pszTitle	= strAlias;
	items[2].pszPageLabel	= strAliasL;

	char strSearch[255];
	char strSearchL[255];
	LoadString(m_hResInst, IDS_SEARCH, strSearch, 255);
	LoadString(m_hResInst, IDS_L_ENGINES, strSearchL, 255);
	items[3].hInstance	= g_hInst;
	items[3].imgID		= IDR_SEARCH;
	items[3].pageID		= PAGE_SEARCH;
	items[3].pfnDlgProc = (DLGPROC) CTrueDialog::WndProc;
	items[3].lParam		= (LPARAM) &m_pageSearch;
	items[3].pResource	= m_pageSearch.GetTemplate(m_hResInst);
	items[3].pszTitle	= strSearch;
	items[3].pszPageLabel	= strSearchL;

	char strAbout[255];
	char strAboutL[255];
	LoadString(m_hResInst, IDS_ABOUT, strAbout, 255);
	LoadString(m_hResInst, IDS_L_ABOUT, strAboutL, 255);
	items[4].hInstance	= g_hInst;
	items[4].imgID		= IDR_ABOUT;
	items[4].pageID		= PAGE_ABOUT;
	items[4].pfnDlgProc = (DLGPROC) CTrueDialog::WndProc;
	items[4].lParam		= (LPARAM) &m_pageAbout;
	items[4].pResource	= m_pageAbout.GetTemplate(m_hResInst);
	items[4].pszTitle	= strAbout;
	items[4].pszPageLabel	= strAboutL;

	SendDlgItemMessage(m_hWnd, IDC_TABS, TTABM_ADDITEM, sizeof(items) / sizeof(items[0]), (LPARAM) items);

	RECT rcPages;
	GetWindowRect(GetDlgItem(m_hWnd, IDC_PAGES), &rcPages);
	MapWindowPoints(NULL, m_hWnd, (LPPOINT) &rcPages, 2);

	TTAB_OPTIONS pp;
	pp.hWndPageLabel = GetDlgItem(m_hWnd, IDC_PAGELABEL);
	pp.rcPage = rcPages;

	SendDlgItemMessage(m_hWnd, IDC_TABS, TTABM_SETPARAMS, NULL, (LPARAM) &pp);
	SendDlgItemMessage(m_hWnd, IDC_TABS, TTABM_ACTIVATETAB, m_tab, 0);
}

CAboutDlg::CAboutDlg(void) : CTrueDialog(IDD_ABOUT)
{
	img.LoadResource(FindResource(g_hInst, MAKEINTRESOURCE(IDR_LOGO), TEXT("PNG")), CXIMAGE_FORMAT_PNG, g_hInst);
}

CAboutDlg::~CAboutDlg(void)
{
}

CAliasDlg::CAliasDlg(void) : CTrueDialog(IDD_ALIAS)
{
	m_sortType = 1;
	m_sortCol = 0;
	m_hImageList = NULL;
}

CAliasDlg::~CAliasDlg(void)
{
	for(int i=0; i < m_aliases.GetCount(); i++)
	{
		if(m_aliases[i].alias) delete m_aliases[i].alias;
		if(m_aliases[i].path) delete m_aliases[i].path;
		if(m_aliases[i].workdir) delete m_aliases[i].workdir;
	}
	m_aliases.Clear();
	if(m_hImageList)
	{
		ImageList_Destroy(m_hImageList);
	}
}

LRESULT CAliasDlg::OnOtherMessage(UINT uMessage, WPARAM wParam, LPARAM lParam, BOOL *ret)
{
	switch(uMessage)
	{
	case WM_SIZE:
		{
			RECT rcClient;
			GetClientRect(m_hWnd, &rcClient);

			RECT rcLST;
			GetWindowRect(GetDlgItem(m_hWnd, IDC_LST), &rcLST);
			MapWindowPoints(NULL, m_hWnd, (LPPOINT) &rcLST, 2);

			SetWindowPos(GetDlgItem(m_hWnd, IDC_LST), NULL, 0, 0, 
				rcClient.right - rcClient.left,
				rcClient.bottom - rcClient.top - rcLST.top, 
				SWP_NOMOVE | SWP_NOZORDER);

			RECT rcBTN;
			GetWindowRect(GetDlgItem(m_hWnd, IDC_FIND), &rcBTN);
			MapWindowPoints(NULL, m_hWnd, (LPPOINT) &rcBTN, 2);
			SetWindowPos(GetDlgItem(m_hWnd, IDC_FIND), NULL, 
				rcClient.right - (rcBTN.right - rcBTN.left), rcBTN.top, 
				0, 0, SWP_NOSIZE | SWP_NOZORDER);
		}
		break;
	}
	return CTrueDialog::OnOtherMessage(uMessage, wParam, lParam, ret);
}


CInetSearchDlg::CInetSearchDlg(void) : CTrueDialog(IDD_SEARCH)
{
	m_sortType = 1;
	m_sortCol = 0;
	m_hImageList = NULL;
}

CInetSearchDlg::~CInetSearchDlg(void)
{
	for(int i=0; i < m_engines.GetCount(); i++)
	{
		if(m_engines[i].encoding)	delete m_engines[i].encoding;
		if(m_engines[i].name)		delete m_engines[i].name;
		if(m_engines[i].url)		delete m_engines[i].url;
		if(m_engines[i].prefix)		delete m_engines[i].prefix;
	}
	m_engines.Clear();
	if(m_hImageList)
	{
		ImageList_Destroy(m_hImageList);
	}
}

void CInetSearchDlg::OnInitDialog(void)
{
	HWND hList = GetDlgItem(m_hWnd, IDC_LST);
	ListView_SetExtendedListViewStyle(hList, LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP | LVS_EX_INFOTIP);

	m_hImageList = ImageList_LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_HEADERICONS), 15, 0, CLR_DEFAULT);
	ListView_SetImageList(hList, m_hImageList, LVSIL_SMALL);

	LVCOLUMN lvc;
	ZeroMemory(&lvc, sizeof(lvc));

	char str[255];
	LoadString(m_hResInst, IDS_ENGINE, str, 255);
	lvc.mask = LVCF_TEXT | LVCF_IMAGE | LVCF_WIDTH | LVCF_FMT;
	lvc.fmt = LVCFMT_COL_HAS_IMAGES | LVCFMT_BITMAP_ON_RIGHT;
	lvc.cx = m_colWidth[0];
	lvc.pszText = str;
	ListView_InsertColumn(hList, 0, &lvc);

	LoadString(m_hResInst, IDS_PREFIX, str, 255);
	lvc.cx = m_colWidth[1];
	lvc.pszText = str;
	ListView_InsertColumn(hList, 1, &lvc);

	ListView_SetColumnOrderArray(hList, 2, m_colOrder);

	FillList();
	SetVis();

	if(m_sortType)
	{
		HWND hList = GetDlgItem(m_hWnd, IDC_LST);
		LVCOLUMN lvc;
		ZeroMemory(&lvc, sizeof(lvc));
		lvc.mask = LVCF_IMAGE;
		switch(m_sortType)
		{
		case 1:
			lvc.iImage = 1;
			break;
		case -1:
			lvc.iImage = 2;
			break;
		}
		ListView_SetColumn(hList, m_sortCol, &lvc);
	}
}

void CInetSearchDlg::OnCommand(DWORD itemID)
{
	switch(LOWORD(itemID))
	{
	case IDC_PREDEF:
		OnAddPredefined();
		break;
	case IDC_DELETE:
		OnDelete();
		break;
	case IDC_EDIT:
		OnEdit();
		break;
	case IDC_NEW:
		if(HIWORD(itemID) == BN_CLICKED)
		{
			CSearchEngineDlg dlg;
			if(dlg.DoModal(m_hWnd, g_hInst) == IDOK)
			{
				AddEngine(&dlg.m_engine);
			}
		}
		break;
	}
}

LRESULT CInetSearchDlg::OnOtherMessage(UINT uMessage, WPARAM wParam, LPARAM lParam, BOOL *ret)
{
	switch(uMessage)
	{
	case WM_SIZE:
		{
			RECT rcClient;
			GetClientRect(m_hWnd, &rcClient);

			RECT rcLST;
			GetWindowRect(GetDlgItem(m_hWnd, IDC_LST), &rcLST);
			MapWindowPoints(NULL, m_hWnd, (LPPOINT) &rcLST, 2);

			SetWindowPos(GetDlgItem(m_hWnd, IDC_LST), NULL, 0, 0, 
				rcClient.right - rcClient.left,
				rcClient.bottom - rcClient.top - rcLST.top, 
				SWP_NOMOVE | SWP_NOZORDER);

			RECT rcBTN;
			GetWindowRect(GetDlgItem(m_hWnd, IDC_PREDEF), &rcBTN);
			MapWindowPoints(NULL, m_hWnd, (LPPOINT) &rcBTN, 2);
			SetWindowPos(GetDlgItem(m_hWnd, IDC_PREDEF), NULL, 
				rcClient.right - (rcBTN.right - rcBTN.left), rcBTN.top, 
				0, 0, SWP_NOSIZE | SWP_NOZORDER);
		}
		break;
	}
	return CTrueDialog::OnOtherMessage(uMessage, wParam, lParam, ret);
}
void CAliasDlg::OnInitDialog(void)
{
	HWND hList = GetDlgItem(m_hWnd, IDC_LST);

	m_hImageList = ImageList_LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_HEADERICONS), 15, 0, CLR_DEFAULT);
	ListView_SetImageList(hList, m_hImageList, LVSIL_SMALL);
	ListView_SetExtendedListViewStyle(hList, LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP | LVS_EX_INFOTIP);

	LVCOLUMN lvc;
	ZeroMemory(&lvc, sizeof(lvc));

	char str[255];
	LoadString(m_hResInst, IDS_ALIAS, str, 255);
	lvc.mask = LVCF_TEXT | LVCF_IMAGE | LVCF_WIDTH | LVCF_FMT;
	lvc.cx = m_colWidth[0];
	lvc.pszText = str;
	lvc.fmt = LVCFMT_COL_HAS_IMAGES | LVCFMT_BITMAP_ON_RIGHT;
	ListView_InsertColumn(hList, 0, &lvc);

	LoadString(m_hResInst, IDS_HOTKEY, str, 255);
	lvc.cx = m_colWidth[1];
	lvc.pszText = str;
	ListView_InsertColumn(hList, 1, &lvc);

	LoadString(m_hResInst, IDS_CMDSTRING, str, 255);
	lvc.cx = m_colWidth[2];
	lvc.pszText = str;
	ListView_InsertColumn(hList, 2, &lvc);

	ListView_SetColumnOrderArray(hList, 3, m_colOrder);

	FillList();
	SetVis();

	if(m_sortType)
	{
		HWND hList = GetDlgItem(m_hWnd, IDC_LST);
		LVCOLUMN lvc;
		ZeroMemory(&lvc, sizeof(lvc));
		lvc.mask = LVCF_IMAGE;
		switch(m_sortType)
		{
		case 1:
			lvc.iImage = 1;
			break;
		case -1:
			lvc.iImage = 2;
			break;
		}
		ListView_SetColumn(hList, m_sortCol, &lvc);
	}
}

void CAliasDlg::FillList(void)
{
	HWND hList = GetDlgItem(m_hWnd, IDC_LST);
	LVITEM lvi;
	ZeroMemory(&lvi, sizeof(lvi));
	for(int i=0; i < m_aliases.GetCount(); i++)
	{
		lvi.iItem		= i;
		lvi.iSubItem	= 0;
		lvi.mask		= LVIF_PARAM | LVIF_TEXT;
		lvi.pszText		= m_aliases[i].alias;
		lvi.lParam		= (LPARAM) i;

		int idx = ListView_InsertItem(hList, &lvi);

		char hkName[255];
		GetHKString(m_aliases[i].hotKey, hkName);
		ListView_SetItemText(hList, idx, 1, hkName);

		ListView_SetItemText(hList, idx, 2, m_aliases[i].path);
	}
	ListView_SortItemsEx(hList, CAliasDlg::cmpMessagesFNC, this);
}

void CAliasDlg::OnCommand(DWORD itemID)
{
	switch(LOWORD(itemID))
	{
	case IDC_DELETE:
		OnDelete();
		break;
	case IDC_EDIT:
		OnEdit();
		break;
	case IDC_NEW:
		if(HIWORD(itemID) == BN_CLICKED)
		{
			CNewAliasDlg dlg;
			if(dlg.DoModal(m_hWnd, g_hInst) == IDOK)
			{
				AddAlias(&dlg.m_alias);
			}
		}
		break;
	}
}


CNewAliasDlg::CNewAliasDlg(void) : CTrueDialog(IDD_NEWALLIAS)
{
	ZeroMemory(&m_alias, sizeof(m_alias));
	m_alias.run = SW_SHOWNORMAL;
}

CNewAliasDlg::~CNewAliasDlg(void)
{
	Clear();
}

struct 
{
	UINT	strID;
	UINT	val;
} g_runValues[] =
{
	{IDS_NORMALWIN,		SW_SHOWNORMAL},
	{IDS_MINIMIZEDWIN,	SW_SHOWMINIMIZED},
	{IDS_MAXIMIZEDWIN,	SW_SHOWMAXIMIZED},
	{IDS_HIDDENWIN,		SW_HIDE},
	{0, 0}
};

void CNewAliasDlg::OnInitDialog(void)
{
	for(int i=0; g_runValues[i].strID; i++)
	{
		char str[255];
		LoadString(m_hResInst, g_runValues[i].strID, str, 255);
		int idx = SendDlgItemMessage(m_hWnd, IDC_RUNAS, CB_ADDSTRING, 0, (LPARAM) str);
		SendDlgItemMessage(m_hWnd, IDC_RUNAS, CB_SETITEMDATA, idx, (LPARAM) g_runValues[i].val);
	}
	SetCBData(IDC_RUNAS, m_alias.run);
	if(m_alias.alias)
	{
		SetDlgItemText(m_hWnd, IDC_ALIAS, m_alias.alias);
	}
	if(m_alias.workdir)
	{
		SetDlgItemText(m_hWnd, IDC_WORKDIR, m_alias.workdir);
	}
	if(m_alias.path)
	{
		SetDlgItemText(m_hWnd, IDC_CMDLINE, m_alias.path);
	}
	SendDlgItemMessage(m_hWnd, IDC_HOTKEY, HKM_SETHOTKEY, m_alias.hotKey, 0);
}

void CNewAliasDlg::Clear(void)
{
	if(m_alias.alias) delete m_alias.alias;
	if(m_alias.path) delete m_alias.path;
	if(m_alias.workdir) delete m_alias.workdir;
	ZeroMemory(&m_alias, sizeof(m_alias));
}

BOOL CNewAliasDlg::OnEndDialog(void)
{
	int szAlias = SendDlgItemMessage(m_hWnd, IDC_ALIAS, WM_GETTEXTLENGTH, 0, 0);
	int szPath = SendDlgItemMessage(m_hWnd, IDC_CMDLINE, WM_GETTEXTLENGTH, 0, 0);
	int szWorkdir = SendDlgItemMessage(m_hWnd, IDC_WORKDIR, WM_GETTEXTLENGTH, 0, 0);
	if(!szAlias || !szPath)
	{
		MessageBox(m_hWnd, "Please enter alias name and command line for alias!", "Command Line", MB_OK | MB_ICONEXCLAMATION);
		return FALSE;
	}
	Clear();
	m_alias.alias = new char[szAlias + 1];
	m_alias.path = new char[szPath + 1];
	if(szWorkdir)
	{
		m_alias.workdir = new char[szWorkdir + 1];
		GetDlgItemText(m_hWnd, IDC_WORKDIR, m_alias.workdir, szWorkdir + 1);
	} else
	{
		m_alias.workdir = NULL;
	}

	GetDlgItemText(m_hWnd, IDC_ALIAS, m_alias.alias, szAlias + 1);
	GetDlgItemText(m_hWnd, IDC_CMDLINE, m_alias.path, szPath + 1);

	m_alias.hotKey = (WORD) SendDlgItemMessage(m_hWnd, IDC_HOTKEY, HKM_GETHOTKEY, 0, 0);
	m_alias.run = GetCBData(IDC_RUNAS);

	return TRUE;
}

int CALLBACK CAliasDlg::cmpMessagesFNC(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	CAliasDlg* pThis = (CAliasDlg*) lParamSort;
	HWND hList = GetDlgItem(pThis->m_hWnd, IDC_LST);

	TCHAR str1[MAX_PATH];
	TCHAR str2[MAX_PATH];
	if(pThis->m_sortType > 0)
	{
		ListView_GetItemText(hList, lParam1, pThis->m_sortCol, str1, MAX_PATH);
		ListView_GetItemText(hList, lParam2, pThis->m_sortCol, str2, MAX_PATH);
	} else
	{
		ListView_GetItemText(hList, lParam2, pThis->m_sortCol, str1, MAX_PATH);
		ListView_GetItemText(hList, lParam1, pThis->m_sortCol, str2, MAX_PATH);
	}
	return StrCmpI(str1, str2);
}

int CALLBACK CInetSearchDlg::cmpMessagesFNC(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	CAliasDlg* pThis = (CAliasDlg*) lParamSort;
	HWND hList = GetDlgItem(pThis->m_hWnd, IDC_LST);

	TCHAR str1[MAX_PATH];
	TCHAR str2[MAX_PATH];
	if(pThis->m_sortType > 0)
	{
		ListView_GetItemText(hList, lParam1, pThis->m_sortCol, str1, MAX_PATH);
		ListView_GetItemText(hList, lParam2, pThis->m_sortCol, str2, MAX_PATH);
	} else
	{
		ListView_GetItemText(hList, lParam2, pThis->m_sortCol, str1, MAX_PATH);
		ListView_GetItemText(hList, lParam1, pThis->m_sortCol, str2, MAX_PATH);
	}
	return StrCmpI(str1, str2);
}

void CAliasDlg::AddAlias(ALIAS* alias)
{
	HWND hList = GetDlgItem(m_hWnd, IDC_LST);

	LVITEM lvi;
	ZeroMemory(&lvi, sizeof(lvi));

	ALIAS alNew;
	alNew.alias = new char[strlen(alias->alias) + 1];
	strcpy(alNew.alias, alias->alias);
	alNew.path = new char[strlen(alias->path) + 1];
	strcpy(alNew.path, alias->path);
	if(alias->workdir)
	{
		alNew.workdir = new char[strlen(alias->workdir) + 1];
		strcpy(alNew.workdir, alias->workdir);
	} else
	{
		alNew.workdir = NULL;
	}
	alNew.hotKey = alias->hotKey;
	alNew.run = alias->run;

	m_aliases.Add(&alNew);

	lvi.iItem		= m_aliases.GetCount() - 1;
	lvi.iSubItem	= 0;
	lvi.mask		= LVIF_PARAM | LVIF_TEXT;
	lvi.pszText		= alNew.alias;
	lvi.lParam		= (LPARAM) (m_aliases.GetCount() - 1);

	int idx = ListView_InsertItem(hList, &lvi);

	char hkName[255];
	GetHKString(alNew.hotKey, hkName);
	ListView_SetItemText(hList, idx, 1, hkName);

	ListView_SetItemText(hList, idx, 2, alNew.path);
	SetVis();
	ListView_SortItemsEx(hList, CAliasDlg::cmpMessagesFNC, this);
}

CSearchEngineDlg::CSearchEngineDlg(void) : CTrueDialog(IDD_NEWSEARCH)
{
	ZeroMemory(&m_engine, sizeof(m_engine));
}

CSearchEngineDlg::~CSearchEngineDlg(void)
{
	Clear();
}

void CSearchEngineDlg::Clear(void)
{
	m_engine.Clear();
}

void CSearchEngineDlg::OnInitDialog(void)
{
	if(m_engine.encoding)
	{
		SetDlgItemText(m_hWnd, IDC_ENCODING, m_engine.encoding);
	}
	if(m_engine.name)
	{
		SetDlgItemText(m_hWnd, IDC_NAME, m_engine.name);
	}
	if(m_engine.url)
	{
		SetDlgItemText(m_hWnd, IDC_URL, m_engine.url);
	}
	if(m_engine.prefix)
	{
		SetDlgItemText(m_hWnd, IDC_PREFIX, m_engine.prefix);
	}
}

BOOL CSearchEngineDlg::OnEndDialog(void)
{
	Clear();
	int sz = SendDlgItemMessage(m_hWnd, IDC_NAME, WM_GETTEXTLENGTH, 0, 0);
	m_engine.name = new char[sz + 1];
	GetDlgItemText(m_hWnd, IDC_NAME, m_engine.name, sz + 1);

	sz = SendDlgItemMessage(m_hWnd, IDC_PREFIX, WM_GETTEXTLENGTH, 0, 0);
	m_engine.prefix = new char[sz + 1];
	GetDlgItemText(m_hWnd, IDC_PREFIX, m_engine.prefix, sz + 1);

	sz = SendDlgItemMessage(m_hWnd, IDC_URL, WM_GETTEXTLENGTH, 0, 0);
	m_engine.url = new char[sz + 1];
	GetDlgItemText(m_hWnd, IDC_URL, m_engine.url, sz + 1);

	sz = SendDlgItemMessage(m_hWnd, IDC_ENCODING, WM_GETTEXTLENGTH, 0, 0);
	m_engine.encoding = new char[sz + 1];
	GetDlgItemText(m_hWnd, IDC_ENCODING, m_engine.encoding, sz + 1);

	return TRUE;
}

void CInetSearchDlg::FillList(void)
{
	HWND hList = GetDlgItem(m_hWnd, IDC_LST);
	LVITEM lvi;
	ZeroMemory(&lvi, sizeof(lvi));
	for(int i=0; i < m_engines.GetCount(); i++)
	{
		lvi.iItem		= i;
		lvi.iSubItem	= 0;
		lvi.mask		= LVIF_PARAM | LVIF_TEXT;
		lvi.pszText		= m_engines[i].name;
		lvi.lParam		= (LPARAM) i;

		int idx = ListView_InsertItem(hList, &lvi);

		ListView_SetItemText(hList, idx, 1, m_engines[i].prefix);
	}
	ListView_SortItemsEx(hList, CInetSearchDlg::cmpMessagesFNC, this);
}

void CInetSearchDlg::AddEngine(SEARCH_ENGINE* engine)
{
	HWND hList = GetDlgItem(m_hWnd, IDC_LST);

	LVITEM lvi;
	ZeroMemory(&lvi, sizeof(lvi));

	SEARCH_ENGINE seNew;
	ZeroMemory(&seNew, sizeof(seNew));
	seNew.Set(*engine);

	m_engines.Add(&seNew);

	lvi.iItem		= m_engines.GetCount() - 1;
	lvi.iSubItem	= 0;
	lvi.mask		= LVIF_PARAM | LVIF_TEXT;
	lvi.pszText		= seNew.name;
	lvi.lParam		= (LPARAM) m_engines.GetCount() - 1;

	int idx = ListView_InsertItem(hList, &lvi);

	ListView_SetItemText(hList, idx, 1, seNew.prefix);
	SetVis();
	ListView_SortItemsEx(hList, CInetSearchDlg::cmpMessagesFNC, this);
}

void CSearchEngineDlg::OnCommand(DWORD itemID)
{
	switch(LOWORD(itemID))
	{
	case IDC_ENCODINGBTN:
		if(HIWORD(itemID) == BN_CLICKED)
		{
			HMENU hMenu = CreatePopupMenu();
			CTArray<ENCODING> encodings;
			IMultiLanguage* ml = NULL;
			HRESULT hr = CoCreateInstance(CLSID_CMultiLanguage, NULL, CLSCTX_INPROC_SERVER, IID_IMultiLanguage, (LPVOID*) &ml);	
			if(hr == S_OK)
			{
				IEnumCodePage* cpEnum = NULL;
				if(ml->EnumCodePages(MIMECONTF_BROWSER, &cpEnum) == S_OK)
				{
					MIMECPINFO cp;
					ULONG ulFetched = 0;
					while(cpEnum->Next(1, &cp, &ulFetched) == S_OK)
					{
						ENCODING enc;
						WideCharToMultiByte(CP_ACP, 0, cp.wszDescription, -1, enc.name, sizeof(enc.name), NULL, NULL);
						WideCharToMultiByte(CP_ACP, 0, cp.wszWebCharset, -1, enc.encoding, sizeof(enc.encoding), NULL, NULL);
						encodings.Add(&enc);
						InsertMenu(hMenu, -1, MF_BYPOSITION | MF_STRING, encodings.GetCount(), enc.name);
					}
					cpEnum->Release();
				}

				ml->Release();
			}
			RECT rcBTN;
			GetWindowRect(GetDlgItem(m_hWnd, IDC_ENCODINGBTN), &rcBTN);
			int ret = TrackPopupMenu(hMenu, TPM_RETURNCMD | TPM_LEFTALIGN | TPM_TOPALIGN, rcBTN.left, rcBTN.bottom, NULL, m_hWnd, NULL);
			if(ret > 0)
			{
				SetDlgItemText(m_hWnd, IDC_ENCODING, encodings[ret - 1].encoding);
			}
			DestroyMenu(hMenu);
		}
		break;
	}
}

BOOL CCmdProperties::OnEndDialog(void)
{
	SendDlgItemMessage(m_hWnd, IDC_TABS, TTABM_APPLY, 0, NULL);
	m_tab = SendDlgItemMessage(m_hWnd, IDC_TABS, TTABM_GETCURRENT, 0, 0);
	return TRUE;
}

void CAliasDlg::SetVis(void)
{
	HWND hList = GetDlgItem(m_hWnd, IDC_LST);

	BOOL hasFocused = FALSE;
	BOOL hasSelected = FALSE;

	int sz = ListView_GetItemCount(hList);
	for(int i=0; i < sz; i++)
	{
		LVITEM lvi;
		ZeroMemory(&lvi, sizeof(lvi));
		lvi.iItem = i;
		lvi.iSubItem = 0;
		lvi.mask = LVIF_STATE;
		lvi.stateMask = LVIS_SELECTED | LVIS_FOCUSED;
		ListView_GetItem(hList, &lvi);

		if(lvi.state & LVIS_FOCUSED)
		{
			hasFocused = TRUE;
		}
		if(lvi.state & LVIS_SELECTED)
		{
			hasSelected = TRUE;
		}
	}
	EnableWindow(GetDlgItem(m_hWnd, IDC_DELETE), hasSelected);
	EnableWindow(GetDlgItem(m_hWnd, IDC_EDIT), hasFocused);
}

DWORD CAliasDlg::OnNotify(int idCtrl, LPNMHDR pnmh)
{
	switch(pnmh->code)
	{
	case PSN_APPLY:
		{
			HWND hList = GetDlgItem(m_hWnd, IDC_LST);
			ListView_GetColumnOrderArray(hList, 3, m_colOrder);
			m_colWidth[0] = ListView_GetColumnWidth(hList, 0);
			m_colWidth[1] = ListView_GetColumnWidth(hList, 1);
			m_colWidth[2] = ListView_GetColumnWidth(hList, 2);
		}
		return PSNRET_NOERROR;
	}

	switch(pnmh->idFrom)
	{
	case IDC_LST:
		{
			switch(pnmh->code)
			{
			case NM_DBLCLK:
				OnEdit();
				break;
			case LVN_COLUMNCLICK:
				{
					LPNMLISTVIEW lv = (LPNMLISTVIEW) pnmh;
					OnColumnClick(lv->iSubItem);
				}
				break;
			case NM_RCLICK:
				break;
			case LVN_ITEMCHANGED:
				SetVis();
				break;
			}
		}
		break;
	}
	return 0;
}

void CAliasDlg::OnColumnClick(int conID)
{
	HWND hWndMsgList = GetDlgItem(m_hWnd, IDC_LST);
	LVCOLUMN lvc;
	ZeroMemory(&lvc, sizeof(lvc));
	lvc.mask = LVCF_IMAGE;
	if(conID == m_sortCol)
	{
		switch(m_sortType)
		{
		case 0:
			m_sortType = 1;
			lvc.iImage = 1;
			break;
		case 1:
			m_sortType = -1;
			lvc.iImage = 2;
			break;
		case -1:
			m_sortType = 1;
			lvc.iImage = 1;
			break;
		}
		ListView_SetColumn(hWndMsgList, conID, &lvc);
	} else
	{
		lvc.iImage = 0;
		ListView_SetColumn(hWndMsgList, m_sortCol, &lvc);
		m_sortType = 1;
		m_sortCol = conID;
		lvc.iImage = 1;
		ListView_SetColumn(hWndMsgList, m_sortCol, &lvc);
	}
	ListView_SortItemsEx(hWndMsgList, CAliasDlg::cmpMessagesFNC, this);
}

void CInetSearchDlg::OnColumnClick(int conID)
{
	HWND hWndMsgList = GetDlgItem(m_hWnd, IDC_LST);
	LVCOLUMN lvc;
	ZeroMemory(&lvc, sizeof(lvc));
	lvc.mask = LVCF_IMAGE;
	if(conID == m_sortCol)
	{
		switch(m_sortType)
		{
		case 0:
			m_sortType = 1;
			lvc.iImage = 1;
			break;
		case 1:
			m_sortType = -1;
			lvc.iImage = 2;
			break;
		case -1:
			m_sortType = 1;
			lvc.iImage = 1;
			break;
		}
		ListView_SetColumn(hWndMsgList, conID, &lvc);
	} else
	{
		lvc.iImage = 0;
		ListView_SetColumn(hWndMsgList, m_sortCol, &lvc);
		m_sortType = 1;
		m_sortCol = conID;
		lvc.iImage = 1;
		ListView_SetColumn(hWndMsgList, m_sortCol, &lvc);
	}
	ListView_SortItemsEx(hWndMsgList, CInetSearchDlg::cmpMessagesFNC, this);
}


void CInetSearchDlg::SetVis(void)
{
	HWND hList = GetDlgItem(m_hWnd, IDC_LST);

	BOOL hasFocused = FALSE;
	BOOL hasSelected = FALSE;

	int sz = ListView_GetItemCount(hList);
	for(int i=0; i < sz; i++)
	{
		LVITEM lvi;
		ZeroMemory(&lvi, sizeof(lvi));
		lvi.iItem = i;
		lvi.iSubItem = 0;
		lvi.mask = LVIF_STATE;
		lvi.stateMask = LVIS_SELECTED | LVIS_FOCUSED;
		ListView_GetItem(hList, &lvi);

		if(lvi.state & LVIS_FOCUSED)
		{
			hasFocused = TRUE;
		}
		if(lvi.state & LVIS_SELECTED)
		{
			hasSelected = TRUE;
		}
	}
	EnableWindow(GetDlgItem(m_hWnd, IDC_DELETE), hasSelected);
	EnableWindow(GetDlgItem(m_hWnd, IDC_EDIT), hasFocused);
}

DWORD CInetSearchDlg::OnNotify(int idCtrl, LPNMHDR pnmh)
{
	switch(pnmh->code)
	{
	case PSN_APPLY:
		{
			HWND hList = GetDlgItem(m_hWnd, IDC_LST);
			ListView_GetColumnOrderArray(hList, 2, m_colOrder);
			m_colWidth[0] = ListView_GetColumnWidth(hList, 0);
			m_colWidth[1] = ListView_GetColumnWidth(hList, 1);
		}
		return PSNRET_NOERROR;
	}
	switch(pnmh->idFrom)
	{
	case IDC_LST:
		{
			switch(pnmh->code)
			{
			case NM_DBLCLK:
				OnEdit();
				break;
			case LVN_COLUMNCLICK:
				{
					LPNMLISTVIEW lv = (LPNMLISTVIEW) pnmh;
					OnColumnClick(lv->iSubItem);
				}
				break;
			case NM_RCLICK:
				break;
			case LVN_ITEMCHANGED:
				SetVis();
				break;
			}
		}
		break;
	}
	return 0;
}

void CAliasDlg::OnEdit(void)
{
	HWND hList = GetDlgItem(m_hWnd, IDC_LST);
	int idx = ListView_GetSelectionMark(hList);
	LVITEM lvi;
	ZeroMemory(&lvi, sizeof(lvi));
	lvi.iItem = idx;
	lvi.iSubItem = 0;
	lvi.mask = LVIF_PARAM;
	ListView_GetItem(hList, &lvi);

	CNewAliasDlg dlg;
	dlg.m_alias.alias = new char[strlen(m_aliases[lvi.lParam].alias) + 1];
	strcpy(dlg.m_alias.alias, m_aliases[lvi.lParam].alias);

	dlg.m_alias.path = new char[strlen(m_aliases[lvi.lParam].path) + 1];
	strcpy(dlg.m_alias.path, m_aliases[lvi.lParam].path);

	if(m_aliases[lvi.lParam].workdir)
	{
		dlg.m_alias.workdir = new char[strlen(m_aliases[lvi.lParam].workdir) + 1];
		strcpy(dlg.m_alias.workdir, m_aliases[lvi.lParam].workdir);
	} else
	{
		m_aliases[lvi.lParam].workdir = NULL;
	}

	dlg.m_alias.hotKey = m_aliases[lvi.lParam].hotKey;
	dlg.m_alias.run = m_aliases[lvi.lParam].run;

	if(dlg.DoModal(m_hWnd, g_hInst))
	{
		m_aliases[lvi.lParam].Clear();

		m_aliases[lvi.lParam].alias = new char[strlen(dlg.m_alias.alias) + 1];
		strcpy(m_aliases[lvi.lParam].alias, dlg.m_alias.alias);

		m_aliases[lvi.lParam].path = new char[strlen(dlg.m_alias.path) + 1];
		strcpy(m_aliases[lvi.lParam].path, dlg.m_alias.path);

		if(dlg.m_alias.workdir)
		{
			m_aliases[lvi.lParam].workdir = new char[strlen(dlg.m_alias.workdir) + 1];
			strcpy(m_aliases[lvi.lParam].workdir, dlg.m_alias.workdir);
		}

		m_aliases[lvi.lParam].hotKey = dlg.m_alias.hotKey;
		m_aliases[lvi.lParam].run = dlg.m_alias.run;

		ListView_SetItemText(hList, idx, 0, m_aliases[lvi.lParam].alias);
		ListView_SetItemText(hList, idx, 2, m_aliases[lvi.lParam].path);

		char hkName[255];
		GetHKString(m_aliases[lvi.lParam].hotKey, hkName);
		ListView_SetItemText(hList, idx, 1, hkName);
		ListView_SortItemsEx(hList, CAliasDlg::cmpMessagesFNC, this);
	}
}

void CInetSearchDlg::OnEdit(void)
{
	HWND hList = GetDlgItem(m_hWnd, IDC_LST);
	int idx = ListView_GetSelectionMark(hList);
	LVITEM lvi;
	ZeroMemory(&lvi, sizeof(lvi));
	lvi.iItem = idx;
	lvi.iSubItem = 0;
	lvi.mask = LVIF_PARAM;
	ListView_GetItem(hList, &lvi);

	CSearchEngineDlg dlg;
	dlg.m_engine.name = new char[strlen(m_engines[lvi.lParam].name) + 1];
	strcpy(dlg.m_engine.name, m_engines[lvi.lParam].name);

	dlg.m_engine.encoding = new char[strlen(m_engines[lvi.lParam].encoding) + 1];
	strcpy(dlg.m_engine.encoding, m_engines[lvi.lParam].encoding);

	dlg.m_engine.prefix = new char[strlen(m_engines[lvi.lParam].prefix) + 1];
	strcpy(dlg.m_engine.prefix, m_engines[lvi.lParam].prefix);

	dlg.m_engine.url = new char[strlen(m_engines[lvi.lParam].url) + 1];
	strcpy(dlg.m_engine.url, m_engines[lvi.lParam].url);

	if(dlg.DoModal(m_hWnd, g_hInst))
	{
		m_engines[lvi.lParam].Clear();

		m_engines[lvi.lParam].name = new char[strlen(dlg.m_engine.name) + 1];
		strcpy(m_engines[lvi.lParam].name, dlg.m_engine.name);

		m_engines[lvi.lParam].prefix = new char[strlen(dlg.m_engine.prefix) + 1];
		strcpy(m_engines[lvi.lParam].prefix, dlg.m_engine.prefix);

		m_engines[lvi.lParam].url = new char[strlen(dlg.m_engine.url) + 1];
		strcpy(m_engines[lvi.lParam].url, dlg.m_engine.url);

		m_engines[lvi.lParam].encoding = new char[strlen(dlg.m_engine.encoding) + 1];
		strcpy(m_engines[lvi.lParam].encoding, dlg.m_engine.encoding);

		ListView_SetItemText(hList, idx, 0, m_engines[lvi.lParam].name);
		ListView_SetItemText(hList, idx, 1, m_engines[lvi.lParam].prefix);
		ListView_SortItemsEx(hList, CInetSearchDlg::cmpMessagesFNC, this);
	}
}

void CAliasDlg::OnDelete(void)
{
	HWND hList = GetDlgItem(m_hWnd, IDC_LST);
	int sz = ListView_GetItemCount(hList);
	if(!sz) return;
	if(MessageBox(m_hWnd, "Do you want to delete selected aliases", "Command Line", MB_YESNO | MB_ICONQUESTION) == IDYES)
	{
		int* delIDX = new int[sz];
		int delSZ = 0;
		for(int i=0; i < sz; i++)
		{
			LVITEM lvi;
			ZeroMemory(&lvi, sizeof(lvi));
			lvi.iItem = i;
			lvi.iSubItem = 0;
			lvi.mask = LVIF_PARAM | LVIF_STATE;
			lvi.stateMask = LVIS_SELECTED;
			ListView_GetItem(hList, &lvi);
			if(lvi.state & LVIS_SELECTED)
			{
				if(!delSZ)
				{
					delIDX[delSZ] = lvi.lParam;
					delSZ++;
				} else
				{
					int insTO = delSZ - 1;
					if(delIDX[insTO] > lvi.lParam)
					{
						delIDX[delSZ] = lvi.lParam;
					} else
					{
						while(insTO >= 0 && delIDX[insTO] < lvi.lParam)
						{
							delIDX[insTO + 1] = delIDX[insTO];
							delIDX[insTO] = lvi.lParam;
							insTO--;
						}
					}
					delSZ++;
				}
			}
		}
		for(int i=0; i < delSZ; i++)
		{
			m_aliases.Delete(delIDX[i]);
		}
		ListView_DeleteAllItems(hList);
		FillList();
		delete delIDX;
	}
}


void CInetSearchDlg::OnDelete(void)
{
	HWND hList = GetDlgItem(m_hWnd, IDC_LST);
	int sz = ListView_GetItemCount(hList);
	if(!sz) return;
	if(MessageBox(m_hWnd, "Do you want to delete selected engines", "Command Line", MB_YESNO | MB_ICONQUESTION) == IDYES)
	{
		int* delIDX = new int[sz];
		int delSZ = 0;
		for(int i=0; i < sz; i++)
		{
			LVITEM lvi;
			ZeroMemory(&lvi, sizeof(lvi));
			lvi.iItem = i;
			lvi.iSubItem = 0;
			lvi.mask = LVIF_PARAM | LVIF_STATE;
			lvi.stateMask = LVIS_SELECTED;
			ListView_GetItem(hList, &lvi);
			if(lvi.state & LVIS_SELECTED)
			{
				if(!delSZ)
				{
					delIDX[delSZ] = lvi.lParam;
					delSZ++;
				} else
				{
					int insTO = delSZ - 1;
					if(delIDX[insTO] > lvi.lParam)
					{
						delIDX[delSZ] = lvi.lParam;
					} else
					{
						while(insTO >= 0 && delIDX[insTO] < lvi.lParam)
						{
							delIDX[insTO + 1] = delIDX[insTO];
							delIDX[insTO] = lvi.lParam;
							insTO--;
						}
					}
					delSZ++;
				}
			}
		}
		for(int i=0; i < delSZ; i++)
		{
			m_engines.Delete(delIDX[i]);
		}
		ListView_DeleteAllItems(hList);
		FillList();
		delete delIDX;
	}
}

void CInetSearchDlg::OnAddPredefined(void)
{
	HMENU hMenu = CreatePopupMenu();
	for(int i=0; defaultEngines[i].name; i++)
	{
		InsertMenu(hMenu, -1, MF_BYPOSITION | MF_STRING, i + 1, defaultEngines[i].name);
	}
	RECT rcBTN;
	GetWindowRect(GetDlgItem(m_hWnd, IDC_PREDEF), &rcBTN);
	int ret = TrackPopupMenu(hMenu, TPM_RETURNCMD | TPM_LEFTALIGN | TPM_TOPALIGN, rcBTN.left, rcBTN.bottom, NULL, m_hWnd, NULL);
	if(ret > 0)
	{
		AddEngine(defaultEngines + ret - 1);
	}
	DestroyMenu(hMenu);
}

void CPropsDlg::FindSkins(void)
{
	int idx = SendDlgItemMessage(m_hWnd, IDC_SKIN, CB_ADDSTRING, 0, (LPARAM) "<default>");
	SendDlgItemMessage(m_hWnd, IDC_SKIN, CB_SETITEMDATA, idx, -1);
	char path[MAX_PATH];
	GetModuleFileName(g_hInst, path, MAX_PATH);
	PathRemoveFileSpec(path);
	PathAddBackslash(path);
	strcat(path, "skins\\");

	WIN32_FIND_DATA fd;
	char findMask[MAX_PATH];
	strcpy(findMask, path);
	strcat(findMask, "*");

	HANDLE hFind = FindFirstFile(findMask, &fd);
	if(hFind != INVALID_HANDLE_VALUE)
	{
		do 
		{
			if(strcmp(fd.cFileName, ".") && strcmp(fd.cFileName, "..") && (fd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
			{
				char SkinName[255];
				char skinFile[MAX_PATH];
				strcpy(skinFile, path);
				strcat(skinFile, fd.cFileName);
				PathAddBackslash(skinFile);
				strcat(skinFile, "cmdlineskin.ini");

				GetPrivateProfileString("options", "name", "", SkinName, 255, skinFile);
				if(SkinName[0])
				{
					SKINS skin;
					GetPrivateProfileString("options", "author", "", skin.author, 255, skinFile);
					GetPrivateProfileString("options", "url", "", skin.url, 255, skinFile);
					GetPrivateProfileString("options", "email", "", skin.email, 255, skinFile);
					strcpy(skin.path, skinFile);
					int idx = SendDlgItemMessage(m_hWnd, IDC_SKIN, CB_ADDSTRING, 0, (LPARAM) SkinName);
					SendDlgItemMessage(m_hWnd, IDC_SKIN, CB_SETITEMDATA, idx, m_skins.GetCount());
					m_skins.Add(&skin);
				}
			}
		} 
		while(FindNextFile(hFind, &fd));
		FindClose(hFind);
	}

	SetCBData(IDC_SKIN, -1);
	char skinPath[MAX_PATH];
	GetFullSkinPath(m_skinPath, skinPath);

	for(int i=0; i < m_skins.GetCount(); i++)
	{
		if(!StrCmpI(m_skins[i].path, skinPath))
		{
			SetCBData(IDC_SKIN, i);
		}
	}
}

void CPropsDlg::SetCBData(DWORD cbID, int data)
{
	int cbCount = SendDlgItemMessage(m_hWnd, cbID, CB_GETCOUNT, 0, 0);
	for(int i=0; i < cbCount; i++)
	{
		int dt = SendDlgItemMessage(m_hWnd, cbID, CB_GETITEMDATA, i, 0);
		if(dt == data)
		{
			SendDlgItemMessage(m_hWnd, cbID, CB_SETCURSEL, i, 0);
			break;
		}
	}
}

DWORD CPropsDlg::GetCBData(DWORD cbID)
{
	int idx = SendDlgItemMessage(m_hWnd, cbID, CB_GETCURSEL, 0, 0);
	return SendDlgItemMessage(m_hWnd, cbID, CB_GETITEMDATA, idx, 0);
}

void CPropsDlg::InitSkin(void)
{
	RECT rcPreview;
	GetWindowRect(GetDlgItem(m_hWnd, IDC_PREVIEW), &rcPreview);

	char curSkin[MAX_PATH];
	curSkin[0] = 0;
	int skinIDX = GetCBData(IDC_SKIN);
	if(skinIDX >=0 && skinIDX < m_skins.GetCount())
	{
		GetRelSkinPath(m_skins[skinIDX].path, curSkin);
		SetDlgItemText(m_hWnd, IDC_AUTHOR, m_skins[skinIDX].author);
		if(m_skins[skinIDX].url[0])
		{
			SendDlgItemMessage(m_hWnd, IDC_AUTHOR, WM_SETURL, 0, (LPARAM) m_skins[skinIDX].url);
		} else
		{
			char url[255];
			sprintf(url, "mailto:%s", m_skins[skinIDX].email);
			SendDlgItemMessage(m_hWnd, IDC_AUTHOR, WM_SETURL, 0, (LPARAM) url);
		}
	} else
	{
		SetDlgItemText(m_hWnd, IDC_AUTHOR, "");
		SendDlgItemMessage(m_hWnd, IDC_AUTHOR, WM_SETURL, 0, (LPARAM) "");
	}
	m_skin.Load(curSkin);
}


void CPropsDlg::OnPaint(void)
{
	PAINTSTRUCT ps;
	BeginPaint(m_hWnd, &ps);
	
	RECT rcPreview;
	GetWindowRect(GetDlgItem(m_hWnd, IDC_PREVIEW), &rcPreview);
	MapWindowPoints(NULL, m_hWnd, (LPPOINT) &rcPreview, 2);

	SIZE sz;
	GetSize(&sz);
	rcPreview.right = rcPreview.left + sz.cx;
	rcPreview.bottom = rcPreview.top + sz.cy;

	m_skin.Draw(ps.hdc, &rcPreview);

	RECT rcText = rcPreview;
	RECT mg;
	m_skin.GetCmdMargins(&mg);
	rcText.left += mg.left;
	rcText.top += mg.top;
	rcText.right += mg.right;
	rcText.bottom += mg.bottom;

	HFONT oldFont = (HFONT) SelectObject(ps.hdc, m_font);
	SetTextColor(ps.hdc, m_skin.GetTextColor());
	SetBkMode(ps.hdc, TRANSPARENT);
	DrawText(ps.hdc, "Notepad.EXE", -1, &rcText, DT_EDITCONTROL);
	SelectObject(ps.hdc, oldFont);

	EndPaint(m_hWnd, &ps);
}

void CPropsDlg::RedrawPreview(void)
{
	RECT rcPreview;
	GetWindowRect(GetDlgItem(m_hWnd, IDC_PREVIEW), &rcPreview);
	MapWindowPoints(NULL, m_hWnd, (LPPOINT) &rcPreview, 2);
	RECT rcClient;
	GetClientRect(m_hWnd, &rcClient);
	rcPreview.right = rcClient.right;
	rcPreview.bottom = rcClient.bottom;

	InvalidateRect(m_hWnd, &rcPreview, TRUE);
}

void CAboutDlg::OnPaint(void)
{
	PAINTSTRUCT ps;
	HDC hdc = BeginPaint(m_hWnd, &ps);
	img.Draw(hdc, 0, 0);
	EndPaint(m_hWnd, &ps);
}

void CAboutDlg::OnInitDialog(void)
{
	char str[MAX_PATH];
	GetModuleFileName(g_hInst, str, MAX_PATH);
	DWORD versionLS, versionMS;
	GetFileVersion(str, versionMS, versionLS);

	char strVer[255];
	Version2Str(versionMS, versionLS, strVer);
	char strText[255];
	wsprintf(strText, "v%s", strVer);

	SetDlgItemText(m_hWnd, IDC_VERSION, strText);
}

void CNewAliasDlg::OnCommand(DWORD itemID)
{
	switch(LOWORD(itemID))
	{
	case IDC_SELECTWD:
		{
			BROWSEINFO bi;
			ZeroMemory(&bi, sizeof(bi));
			bi.hwndOwner = m_hWnd;
			TCHAR displayName[MAX_PATH];
			displayName[0] = 0;
			bi.pszDisplayName = displayName;
			bi.lpszTitle = TEXT("Select the folder where to save the message");
			bi.ulFlags = BIF_USENEWUI | BIF_RETURNONLYFSDIRS;
			LPITEMIDLIST pidl = SHBrowseForFolder(&bi);
			if(pidl)
			{
				SHGetPathFromIDList(pidl, displayName);
				IMalloc* m;
				SHGetMalloc(&m);
				m->Free(pidl);
				m->Release();

				SetDlgItemText(m_hWnd, IDC_WORKDIR, displayName);
			}
		}
		break;
	}
}

void CPropsDlg::CreateCmdFont(void)
{
	char	fontName[32];
	BOOL	fontBold;
	BOOL	fontItalic;
	int		fontSize;

	fontSize = GetDlgItemInt(m_hWnd, IDC_FONTSIZE, NULL, FALSE);
	fontBold = SendDlgItemMessage(m_hWnd, IDC_BOLD, BM_GETCHECK, 0, 0);
	fontItalic = SendDlgItemMessage(m_hWnd, IDC_ITALIC, BM_GETCHECK, 0, 0);
	int idx = SendDlgItemMessage(m_hWnd, IDC_FONTNAME, CB_GETCURSEL, 0, 0);
	if(idx < 0)
	{
		GetDlgItemText(m_hWnd, IDC_FONTNAME, fontName, 32);
	} else
	{
		SendDlgItemMessage(m_hWnd, IDC_FONTNAME, CB_GETLBTEXT, idx, (LPARAM) fontName);
	}


	if(m_font)
	{
		DeleteObject(m_font);
		m_font = NULL;
	}
	if(fontName[0])
	{
		HDC hDC = GetDC(NULL);
		LOGFONT lf;
		ZeroMemory(&lf, sizeof(LOGFONT));
		strcpy(lf.lfFaceName, fontName);
		lf.lfHeight = -MulDiv(fontSize, GetDeviceCaps(hDC, LOGPIXELSY), 72);
		if(fontBold)
		{
			lf.lfWeight = FW_BOLD;
		} else
		{
			lf.lfWeight = FW_NORMAL;
		}
		lf.lfItalic = fontItalic;
		lf.lfCharSet = DEFAULT_CHARSET;
		lf.lfOutPrecision = OUT_DEFAULT_PRECIS;
		lf.lfQuality = DEFAULT_QUALITY;
		m_font = CreateFontIndirect(&lf);
		ReleaseDC(NULL, hDC);
	}
	if(!m_font) 
	{
		NONCLIENTMETRICS ncm;
		ZeroMemory(&ncm, sizeof(ncm));
		ncm.cbSize = sizeof(ncm);
		SystemParametersInfo(SPI_GETNONCLIENTMETRICS, ncm.cbSize, &ncm, 0);
		m_font = CreateFontIndirect(&ncm.lfMessageFont);
	}
	RedrawPreview();
}

void CPropsDlg::GetSize(LPSIZE sz)
{
	SIZE szFont;
	HDC dc = GetDC(m_hWnd);
	HFONT oldFont = (HFONT) SelectObject(dc, m_font);
	GetTextExtentPoint32(dc, "Notepad.EXE", strlen("Notepad.EXE"), &szFont);
	SelectObject(dc, oldFont);
	ReleaseDC(m_hWnd, dc);

	RECT rcMargins;
	m_skin.GetCmdMargins(&rcMargins);

	DWORD len = szFont.cx + rcMargins.left + rcMargins.right;

	sz->cx = len + /*rcMargins.left + rcMargins.right + */2;
	sz->cy = szFont.cy + rcMargins.top + rcMargins.bottom + 1;
}

CHistoryDlg::CHistoryDlg() : CTrueDialog(IDD_HISTORYTAB)
{
}

CHistoryDlg::~CHistoryDlg()
{
}


void CHistoryDlg::OnInitDialog()
{
	for(int i=0; i < m_history->GetCount(); i++)
	{
		SendDlgItemMessage(m_hWnd, IDC_HISTORYLIST, LB_ADDSTRING, 0, (LPARAM) (*m_history)[i]);
	}
	SetVis();
}

LRESULT CHistoryDlg::OnOtherMessage(UINT uMessage, WPARAM wParam, LPARAM lParam, BOOL *ret)
{
	switch(uMessage)
	{
	case WM_SIZE:
		{
			RECT rcClient;
			GetClientRect(m_hWnd, &rcClient);

			RECT rcLST;
			GetWindowRect(GetDlgItem(m_hWnd, IDC_HISTORYLIST), &rcLST);
			MapWindowPoints(NULL, m_hWnd, (LPPOINT) &rcLST, 2);

			SetWindowPos(GetDlgItem(m_hWnd, IDC_HISTORYLIST), NULL, 0, 0, 
				rcClient.right - rcClient.left,
				rcClient.bottom - rcClient.top - rcLST.top, 
				SWP_NOMOVE | SWP_NOZORDER);

/*			RECT rcBTN;
			GetWindowRect(GetDlgItem(m_hWnd, IDC_FIND), &rcBTN);
			MapWindowPoints(NULL, m_hWnd, (LPPOINT) &rcBTN, 2);
			SetWindowPos(GetDlgItem(m_hWnd, IDC_FIND), NULL, 
				rcClient.right - (rcBTN.right - rcBTN.left), rcBTN.top, 
				0, 0, SWP_NOSIZE | SWP_NOZORDER);*/
		}
		break;
	}
	return CTrueDialog::OnOtherMessage(uMessage, wParam, lParam, ret);
}

void CHistoryDlg::OnCommand(DWORD itemID)
{
	switch(LOWORD(itemID))
	{
	case IDC_DELETE:
		OnDelete();
		break;
	case IDC_CLEAR:
		SendDlgItemMessage(m_hWnd, IDC_HISTORYLIST, LB_RESETCONTENT, 0, 0);
		SetVis();
		break;
	case IDC_HISTORYLIST:
		if(HIWORD(itemID) == LBN_SELCHANGE)
		{
			SetVis();
		}
		break;
	}
}

DWORD CHistoryDlg::OnNotify(int idCtrl, LPNMHDR pnmh)
{
	switch(pnmh->code)
	{
	case PSN_APPLY:
		{
			for(int i=0; i < m_history->GetCount(); i++)
			{
				delete (*m_history)[i];
			}
			m_history->Clear();
			int numItems = SendDlgItemMessage(m_hWnd, IDC_HISTORYLIST, LB_GETCOUNT, 0, 0);
			for(int i=0; i < numItems; i++)
			{
				int sz = SendDlgItemMessage(m_hWnd, IDC_HISTORYLIST, LB_GETTEXTLEN, i, 0) + 1;
				char* str = new char[sz];
				SendDlgItemMessage(m_hWnd, IDC_HISTORYLIST, LB_GETTEXT, i, (LPARAM) str);
				m_history->Add(&str);
			}
		}
		return PSNRET_NOERROR;
	}
	return 0;
}

void CHistoryDlg::SetVis(void)
{
	if(SendDlgItemMessage(m_hWnd, IDC_HISTORYLIST, LB_GETCOUNT, 0, 0))
	{
		EnableWindow(GetDlgItem(m_hWnd, IDC_CLEAR), TRUE);
		if(SendDlgItemMessage(m_hWnd, IDC_HISTORYLIST, LB_GETSELCOUNT, 0, 0) > 0)
		{
			EnableWindow(GetDlgItem(m_hWnd, IDC_DELETE), TRUE);
		} else
		{
			EnableWindow(GetDlgItem(m_hWnd, IDC_DELETE), FALSE);
		}
	} else
	{
		EnableWindow(GetDlgItem(m_hWnd, IDC_CLEAR), FALSE);
		EnableWindow(GetDlgItem(m_hWnd, IDC_DELETE), FALSE);
	}
}

void CHistoryDlg::OnDelete(void)
{
	int cnt = SendDlgItemMessage(m_hWnd, IDC_HISTORYLIST, LB_GETSELCOUNT, 0, 0);
	if(cnt > 0)
	{
		while(TRUE)
		{
			int numItems = SendDlgItemMessage(m_hWnd, IDC_HISTORYLIST, LB_GETCOUNT, 0, 0);
			BOOL found = FALSE;
			for(int i = 0; i < numItems; i++)
			{
				if(SendDlgItemMessage(m_hWnd, IDC_HISTORYLIST, LB_GETSEL, i, 0))
				{
					found = TRUE;
					SendDlgItemMessage(m_hWnd, IDC_HISTORYLIST, LB_DELETESTRING, i, 0);
					break;
				}
			}
			if(!found) break;
		}
	}
	SetVis();
}
