// PropsDlg.h: interface for the CPropsDlg class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_PROPSDLG_H__444F3683_8643_47DB_B25F_7C47DA18C020__INCLUDED_)
#define AFX_PROPSDLG_H__444F3683_8643_47DB_B25F_7C47DA18C020__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "TrueDialog.h"
#include "skin.h"

#define PAGE_GENERAL	1
#define PAGE_ALIAS		2
#define PAGE_ABOUT		3
#define PAGE_SEARCH		4
#define PAGE_HISTORY	5

struct SKINS
{
	char name[255];
	char path[MAX_PATH];
	char author[255];
	char url[255];
	char email[255];
};

class CPropsDlg : public CTrueDialog  
{
	CSkin	m_skin;
public:
	DWORD	sz;
	UINT	m_align;
	UINT	m_historySize;
	WORD	m_hotKey;
	BOOL	m_autoHide;
	char	m_skinPath[MAX_PATH];
	BOOL	m_enableASList;
	BOOL	m_enableAUTOAPPEND;

	char	m_fontName[32];
	BOOL	m_fontBold;
	BOOL	m_fontItalic;
	int		m_fontSize;

	CTArray<SKINS> m_skins;

	HFONT	m_font;

	CPropsDlg();
	virtual ~CPropsDlg();
protected:
	DWORD OnNotify(int idCtrl, LPNMHDR pnmh);
	void OnInitDialog();
	void OnCommand(DWORD itemID);
	void OnPaint(void);
private:
	void SetVis(void);
	void FindSkins(void);
	void SetCBData(DWORD cbID, int data);
	DWORD GetCBData(DWORD cbID);
	void InitSkin(void);
	void RedrawPreview(void);
	void GetSize(LPSIZE sz);
	void CreateCmdFont();
};

class CAboutDlg :
	public CTrueDialog
{
	CxImage img;
public:
	CAboutDlg(void);
	~CAboutDlg(void);
protected:
	void OnPaint(void);
	void OnInitDialog(void);
};

class CCmdHistory;

class CHistoryDlg : public CTrueDialog
{
public:
	CCmdHistory*	m_history;

	CHistoryDlg();
	~CHistoryDlg();
protected:
	void OnInitDialog();
	LRESULT OnOtherMessage(UINT uMessage, WPARAM wParam, LPARAM lParam, BOOL *ret);
	void OnCommand(DWORD itemID);
	DWORD OnNotify(int idCtrl, LPNMHDR pnmh);
private:
	void SetVis(void);
	void OnDelete(void);
};

class CAliasDlg : public CTrueDialog
{
	HIMAGELIST		m_hImageList;
public:
	CTArray<ALIAS>	m_aliases;
	int				m_sortType;
	int				m_sortCol;
	int				m_colWidth[3];
	int				m_colOrder[3];

	CAliasDlg(void);
	~CAliasDlg(void);
protected:
	LRESULT OnOtherMessage(UINT uMessage, WPARAM wParam, LPARAM lParam, BOOL *ret);
	void OnInitDialog(void);
	void OnCommand(DWORD itemID);
	DWORD OnNotify(int idCtrl, LPNMHDR pnmh);
private:
	void FillList(void);
	void AddAlias(ALIAS* alias);
	void SetVis(void);
	void OnEdit(void);
	void OnDelete(void);
	void OnColumnClick(int conID);

	static int CALLBACK cmpMessagesFNC(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort);
};

class CInetSearchDlg : public CTrueDialog
{
	HIMAGELIST		m_hImageList;
	static int CALLBACK cmpMessagesFNC(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort);
public:
	CTArray<SEARCH_ENGINE>	m_engines;
	int				m_sortType;
	int				m_sortCol;
	int				m_colWidth[2];
	int				m_colOrder[2];

	CInetSearchDlg(void);
	~CInetSearchDlg(void);
protected:
	LRESULT OnOtherMessage(UINT uMessage, WPARAM wParam, LPARAM lParam, BOOL *ret);
	void OnInitDialog(void);
	void OnCommand(DWORD itemID);
	DWORD OnNotify(int idCtrl, LPNMHDR pnmh);
private:
	void FillList(void);
	void AddEngine(SEARCH_ENGINE* engine);
	void SetVis(void);
	void OnEdit(void);
	void OnDelete();
	void OnAddPredefined(void);
	void OnColumnClick(int conID);
};

class CCmdProperties : public CTrueDialog
{
public:
	CPropsDlg		m_pageGeneral;
	CHistoryDlg		m_pageHistory;
	CAliasDlg		m_pageAlias;
	CInetSearchDlg	m_pageSearch;
	CAboutDlg		m_pageAbout;
	int				m_tab;

	CCmdProperties(void);
	~CCmdProperties(void);
protected:
	void OnInitDialog(void);
	BOOL OnEndDialog(void);
};

class CNewAliasDlg :
	public CTrueDialog
{
public:
	ALIAS	m_alias;

	CNewAliasDlg(void);
	~CNewAliasDlg(void);
protected:
	void OnInitDialog(void);
	void OnCommand(DWORD itemID);
	BOOL OnEndDialog(void);
private:
	void Clear(void);
};

class CSearchEngineDlg :
	public CTrueDialog
{
public:
	SEARCH_ENGINE	m_engine;

	CSearchEngineDlg(void);
	~CSearchEngineDlg(void);
private:
	void Clear(void);
protected:
	void OnInitDialog(void);
	BOOL OnEndDialog(void);
	void OnCommand(DWORD itemID);
};

#endif // !defined(AFX_PROPSDLG_H__444F3683_8643_47DB_B25F_7C47DA18C020__INCLUDED_)
