#include "stdafx.h"
#include "skin.h"
#include "resource.h"
#include ".\skin.h"
#include <shlwapi.h>


void GetFullSkinPath(LPSTR skinPath, LPSTR fullSkinPath)
{
  GetModuleFileName(g_hInst, fullSkinPath, MAX_PATH);
  PathRemoveFileSpec(fullSkinPath);
  PathAddBackslash(fullSkinPath);
  strcat(fullSkinPath, "skins\\");
  strcat(fullSkinPath, skinPath);
}
void GetRelSkinPath(LPSTR fullSkinPath, LPSTR skinPath)
{
  char* str = StrStrI(fullSkinPath, "\\skins\\");
  if(str)
  {
    str += strlen("\\skins\\");
    strcpy(skinPath, str);
  }
}

CSkin::CSkin(void)
{
	m_cmdln = 0;
}

CSkin::~CSkin(void)
{
	Clear();
}

void CSkin::LoadDefault(void)
{
	Clear();
	m_cmdln = new CxImage;
	m_cmdln->LoadResource(FindResource(g_hInst, MAKEINTRESOURCE(IDR_DEFCMDLN), TEXT("PNG")), CXIMAGE_FORMAT_PNG, g_hInst);
	m_cmdlnMargins.left = m_cmdlnMargins.right = m_cmdlnMargins.bottom = m_cmdlnMargins.top = 2;
	m_editMargins.left = m_editMargins.right = 4;
	m_editMargins.bottom = m_editMargins.top = 4;


}

void CSkin::Clear(void)
{
	if(m_cmdln)
	{
		delete m_cmdln;
		m_cmdln = NULL;
	}
	ZeroMemory(&m_cmdlnMargins, sizeof(m_cmdlnMargins));
}

void CSkin::Draw(HDC hdc, LPRECT rc)
{
	if(m_cmdln)
	{
		//left top corner
		if(m_cmdlnMargins.left && m_cmdlnMargins.top)
		{
			CxImage img;
			m_cmdln->Crop(0, 0, m_cmdlnMargins.left, m_cmdlnMargins.top, &img);
			img.Draw(hdc, rc->left, rc->top);
		}
		//top
		{
			CxImage img;
			m_cmdln->Crop(m_cmdlnMargins.left, 0, m_cmdln->GetWidth() - m_cmdlnMargins.right, m_cmdlnMargins.top, &img);
			img.Draw(hdc, rc->left + m_cmdlnMargins.left, rc->top, rc->right - rc->left - m_cmdlnMargins.left - m_cmdlnMargins.right, m_cmdlnMargins.top);
		}
		//right top corner
		if(m_cmdlnMargins.right && m_cmdlnMargins.top)
		{
			CxImage img;
			m_cmdln->Crop(m_cmdln->GetWidth() - m_cmdlnMargins.right, 0, m_cmdln->GetWidth(), m_cmdlnMargins.top, &img);
			img.Draw(hdc, rc->right - m_cmdlnMargins.right, rc->top);
		}
		//////////////////////////////////////////////////////
		//left bottom corner
		if(m_cmdlnMargins.left && m_cmdlnMargins.bottom)
		{
			CxImage img;
			m_cmdln->Crop(0, m_cmdln->GetHeight() - m_cmdlnMargins.bottom, m_cmdlnMargins.left, m_cmdln->GetHeight(), &img);
			img.Draw(hdc, rc->left, rc->bottom - m_cmdlnMargins.bottom);
		}
		//bottom
		{
			CxImage img;
			m_cmdln->Crop(m_cmdlnMargins.left, m_cmdln->GetHeight() - m_cmdlnMargins.bottom, m_cmdln->GetWidth() - m_cmdlnMargins.right, m_cmdln->GetHeight(), &img);
			img.Draw(hdc, rc->left + m_cmdlnMargins.left, rc->bottom - m_cmdlnMargins.bottom, rc->right - rc->left - m_cmdlnMargins.left - m_cmdlnMargins.right);
		}
		//right bottom corner
		if(m_cmdlnMargins.right && m_cmdlnMargins.top)
		{
			CxImage img;
			m_cmdln->Crop(m_cmdln->GetWidth() - m_cmdlnMargins.right, m_cmdln->GetHeight() - m_cmdlnMargins.bottom, m_cmdln->GetWidth(), m_cmdln->GetHeight(), &img);
			img.Draw(hdc, rc->right - m_cmdlnMargins.right, rc->bottom - m_cmdlnMargins.bottom);
		}
		//////////////////////////////////////////////////////
		//left center corner
		if(m_cmdlnMargins.left && m_cmdlnMargins.bottom)
		{
			CxImage img;
			m_cmdln->Crop(0, m_cmdlnMargins.top, m_cmdlnMargins.left, m_cmdln->GetHeight() - m_cmdlnMargins.bottom, &img);
			img.Draw(hdc, rc->left, rc->top + m_cmdlnMargins.top, -1, rc->bottom - rc->top - m_cmdlnMargins.top - m_cmdlnMargins.bottom);
		}
		//center
		{
			CxImage img;
			m_cmdln->Crop(m_cmdlnMargins.left, m_cmdlnMargins.top, m_cmdln->GetWidth() - m_cmdlnMargins.right, m_cmdln->GetHeight() - m_cmdlnMargins.bottom, &img);
			img.Draw(hdc, rc->left + m_cmdlnMargins.left, rc->top + m_cmdlnMargins.top, rc->right - rc->left - m_cmdlnMargins.left - m_cmdlnMargins.right, rc->bottom - rc->top - m_cmdlnMargins.top - m_cmdlnMargins.bottom);
		}
		//right center corner
		if(m_cmdlnMargins.right && m_cmdlnMargins.top)
		{
			CxImage img;
			m_cmdln->Crop(m_cmdln->GetWidth() - m_cmdlnMargins.right, m_cmdlnMargins.top, m_cmdln->GetWidth(), m_cmdln->GetHeight() - m_cmdlnMargins.bottom, &img);
			img.Draw(hdc, rc->right - m_cmdlnMargins.right, rc->top + m_cmdlnMargins.top, -1, rc->bottom - rc->top - m_cmdlnMargins.top - m_cmdlnMargins.bottom);
		}
	}
}

void CSkin::Load(char* path)
{
	if(!path || path && !path[0])
	{
		LoadDefault();
	} else
	{
		char fileName[MAX_PATH];

		GetModuleFileName(g_hInst, fileName, MAX_PATH);
		PathRemoveFileSpec(fileName);
		PathAddBackslash(fileName);
		strcat(fileName, "skins\\");
		strcat(fileName, path);

		if(!PathFileExists(fileName))
		{
			LoadDefault();
			return;
		}
		Clear();

		char skinPath[MAX_PATH];
		strcpy(skinPath, fileName);
		PathRemoveFileSpec(skinPath);
		PathAddBackslash(skinPath);

		char flName[MAX_PATH];

		// loading background
		char str[MAX_PATH];
		GetPrivateProfileString("cmdline", "background", "", str, MAX_PATH, fileName);
		strcpy(flName, skinPath);
		strcat(flName, str);
		m_cmdln = new CxImage;
		m_cmdln->Load(flName, CXIMAGE_FORMAT_PNG);
		if(!m_cmdln->IsValid())
		{
			delete m_cmdln;
			m_cmdln = NULL;
		}
		GetPrivateProfileString("cmdline", "sizingMargins", "0 0 0 0", str, MAX_PATH, fileName);
		Str2Rect(str, &m_cmdlnMargins);

		GetPrivateProfileString("cmdline", "editMargins", "0 0 0 0", str, MAX_PATH, fileName);
		Str2Rect(str, &m_editMargins);

		GetPrivateProfileString("cmdline", "textColor", "0 0 0", str, MAX_PATH, fileName);
		m_textColor = ParseColor(str);
	}
}

void CSkin::Str2Rect(char *str, LPRECT rc)
{
	ZeroMemory(rc, sizeof(RECT));
	char* stop;
	rc->left = strtoul(str, &stop, 10);
	if(stop)
	{
		rc->top = strtoul(stop, &stop, 10);
	}
	if(stop)
	{
		rc->right = strtoul(stop, &stop, 10);
	}
	if(stop)
	{
		rc->bottom = strtoul(stop, &stop, 10);
	}
}

COLORREF CSkin::ParseColor(char* str)
{
	DWORD r = 0;
	DWORD g = 0;
	DWORD b = 0;
	char strVal[50];
	char* curPos = str;
	while(curPos[0] && !isdigit(curPos[0])) curPos++;
	int i=0;
	while(curPos[0] && isdigit(curPos[0])) 
	{
		strVal[i] = curPos[0];
		i++;
		curPos++;
	}
	strVal[i] = 0;
	r = atoi(strVal);

	while(curPos[0] && !isdigit(curPos[0])) curPos++;

	i=0;
	while(curPos[0] && isdigit(curPos[0])) 
	{
		strVal[i] = curPos[0];
		i++;
		curPos++;
	}
	strVal[i] = 0;
	g = atoi(strVal);

	while(curPos[0] && !isdigit(curPos[0])) curPos++;

	i=0;
	while(curPos[0] && isdigit(curPos[0])) 
	{
		strVal[i] = curPos[0];
		i++;
		curPos++;
	}
	strVal[i] = 0;
	b = atoi(strVal);

	return RGB(r, g, b);
}
