#include <windows.h>
#include <windowsx.h>
#include "treetabctl.h"
#include ".\treetabctl.h"

void RegisterTreeTabClass(HINSTANCE hInst)
{
	WNDCLASS wc;
	if(!GetClassInfo(hInst, TREETAB_CLASS, &wc))
	{
		ZeroMemory(&wc, sizeof(wc));
		wc.style          = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
		wc.lpfnWndProc    = (WNDPROC)CTreeTabCtl::WndProc;
		wc.cbClsExtra     = 0;
		wc.cbWndExtra     = 0;
		wc.hInstance      = hInst;
		wc.hIcon          = NULL;
		wc.hCursor        = LoadCursor(NULL, IDC_ARROW);
		wc.hbrBackground  = NULL;//(HBRUSH)(COLOR_WINDOW + 1);
		wc.lpszMenuName   = NULL;
		wc.lpszClassName  = TREETAB_CLASS;

		RegisterClass(&wc);
	}
	if(!GetClassInfo(hInst, TABLABEL_CLASS, &wc))
	{
		ZeroMemory(&wc, sizeof(wc));
		wc.style          = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
		wc.lpfnWndProc    = (WNDPROC)CTabLabel::WndProc;
		wc.cbClsExtra     = 0;
		wc.cbWndExtra     = 0;
		wc.hInstance      = hInst;
		wc.hIcon          = NULL;
		wc.hCursor        = LoadCursor(NULL, IDC_ARROW);
		wc.hbrBackground  = (HBRUSH)(COLOR_HIGHLIGHT + 1);
		wc.lpszMenuName   = NULL;
		wc.lpszClassName  = TABLABEL_CLASS;

		RegisterClass(&wc);
	}
}

CTreeTabCtl::CTreeTabCtl(void)
{
	m_hWnd	= NULL;
	m_items = NULL;
	m_count = 0;
	m_iconSize.cx = 0;
	m_iconSize.cy = 0;
	m_pagesLabel = NULL;
}

CTreeTabCtl::~CTreeTabCtl(void)
{
	for(int i=0; i < m_count; i++)
	{
		if(m_items[i].img) delete m_items[i].img;
		if(m_items[i].pszTitle) delete m_items[i].pszTitle;
	}
	delete m_items;
	m_items = NULL;
	m_count = 0;
}

LRESULT CALLBACK CTreeTabCtl::WndProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam)
{
	CTreeTabCtl* pThis = NULL;
	if(IsWindow(hWnd))
	{
		pThis = (CTreeTabCtl*)GetProp(hWnd, TEXT("CTreeTabCtl_this"));
	}

	if(pThis || uMessage == WM_CREATE)
	{
		switch (uMessage)
		{
		case TTABM_UPDATETEMPLATES:
			{
				LPDLGTEMPLATE* items = (LPDLGTEMPLATE*) lParam;
				for(int i=0; i < pThis->m_count; i++)
				{
					pThis->m_items[i].pResource = items[i];
				}
			}
			break;
		case TTABM_REINITTABS:
			{
				for(int i=0; i < pThis->m_count; i++)
				{
					if(IsWindow(pThis->m_items[i].hWndPage))
					{
						SendMessage(pThis->m_items[i].hWndPage, WM_INITDIALOG, 0, pThis->m_items[i].lParam);
					}
				}
			}
			return 0;
		case WM_KEYDOWN:
			pThis->OnKeyDown(wParam);
			return 0;
		case WM_GETDLGCODE:
			return DLGC_WANTARROWS;
		case WM_KILLFOCUS:
		case WM_SETFOCUS:
			InvalidateRect(pThis->m_hWnd, NULL, FALSE);
			break;
		case TTABM_GETCURRENT:
			{
				for(int i=0; i < pThis->m_count; i++)
				{
					if(pThis->m_items[i].state & TTAB_STATE_ACTIVE)
					{
						return pThis->m_items[i].pageID;
					}
				}
			}
			return -1;
		case TTABM_APPLY:
			pThis->Apply();
			return 0;
		case TTABM_ACTIVATETAB:
			for(int i=0; i < pThis->m_count; i++)
			{
				if(pThis->m_items[i].pageID == (UINT) wParam)
				{
					pThis->ActivateItem(i);
					return 0;
				}
			}
			return -1;
		case TTABM_SETPARAMS:
			pThis->SetParams((TTAB_OPTIONS*) lParam);
			pThis->m_pagesRect = *((LPRECT) lParam);
			return 0;
		case TTABM_ADDITEM:
			pThis->InitItems((TTAB_PAGE*) lParam, wParam);
			return 0;
		case WM_LBUTTONDOWN:
			pThis->OnLButtonDown(GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));
			SetFocus(pThis->m_hWnd);
			return 0;
		case WM_CREATE:
			{
				LPCREATESTRUCT lpcs = (LPCREATESTRUCT)lParam;
				pThis = new CTreeTabCtl;
				SetProp(hWnd, TEXT("CTreeTabCtl_this"), (HANDLE) pThis);
				pThis->m_hWnd = hWnd;
				pThis->OnCreate();
			}
			break;
		case WM_DESTROY:
			delete pThis;
			RemoveProp(hWnd, TEXT("CTreeTabCtl_this"));
			break;
		case WM_PAINT:
			{
				PAINTSTRUCT ps;
				HDC hdc = BeginPaint(pThis->m_hWnd, &ps);
				HFONT fnt = (HFONT) SendMessage(GetParent(pThis->m_hWnd), WM_GETFONT, 0, 0);
				HFONT oldfnt = (HFONT) SelectObject(ps.hdc, fnt);
				pThis->OnPaint(hdc);
				SelectObject(ps.hdc, oldfnt);
				EndPaint(pThis->m_hWnd, &ps);
			}
			return 0;
		}
	}

	return DefWindowProc(hWnd, uMessage, wParam, lParam);
}

void CTreeTabCtl::OnCreate(void)
{
}

void CTreeTabCtl::OnLButtonDown(int x, int y)
{
	POINT pt;
	pt.x = x; pt.y = y;
	for(int i=0; i < m_count; i++)
	{
		if(PtInRect(&m_items[i].rcItem, pt))
		{
			ActivateItem(i);
			return;
		}
	}
}

void CTreeTabCtl::OnPaint(HDC hdc)
{
	HDC memDC = CreateCompatibleDC(hdc);
	HBITMAP bmp = CreateCompatibleBitmap(hdc, m_itemSize.cx, m_itemSize.cy);
	HBITMAP oldBitmap = (HBITMAP) SelectObject(memDC, bmp);
	HFONT fnt = (HFONT) SendMessage(GetParent(m_hWnd), WM_GETFONT, 0, 0);
	HFONT oldFont = (HFONT) SelectObject(memDC, fnt);
	int y = 0;
	RECT rcClient;
	GetClientRect(m_hWnd, &rcClient);
	for(int i=0; i < m_count; i++)
	{
		RECT rcButton = {0, 0, 0, 0};
		rcButton.right	= m_itemSize.cx;
		rcButton.bottom = m_itemSize.cy;
		DrawItem(memDC, &m_items[i], &rcButton);
		BitBlt(hdc, m_items[i].rcItem.left, m_items[i].rcItem.top,
			m_itemSize.cx,
			m_itemSize.cy,
			memDC, 0, 0, SRCCOPY);
		y = m_items[i].rcItem.bottom;
	}
	SelectObject(memDC, oldBitmap);
	SelectObject(memDC, oldFont);
	DeleteDC(memDC);
	DeleteObject(bmp);
	if(y < rcClient.bottom)
	{
		rcClient.top = y;
		FillRect(hdc, &rcClient, (HBRUSH) (COLOR_WINDOW + 1));
	}
}

void CTreeTabCtl::InitItems(TTAB_PAGE* items, int count)
{
	m_iconSize.cx = 0;
	m_iconSize.cy = 0;
	m_textSize.cx = 0;
	m_textSize.cy = 0;
	m_items = new _TTAB_PAGE[count];
	HDC dc = GetDC(m_hWnd);
	HFONT fnt = (HFONT) SendMessage(GetParent(m_hWnd), WM_GETFONT, 0, 0);
	HFONT oldfnt = (HFONT) SelectObject(dc, fnt);
	for(int i=0; i < count; i++)
	{
		m_items[i].hInstance	= items[i].hInstance;
		m_items[i].lParam		= items[i].lParam;
		m_items[i].pageID		= items[i].pageID;
		m_items[i].pfnDlgProc	= items[i].pfnDlgProc;
		m_items[i].pResource	= items[i].pResource;
		m_items[i].state		= 0;
		m_items[i].hWndPage		= NULL;
		m_items[i].pszTitle		= new TCHAR[lstrlen(items[i].pszTitle) + 1];
		lstrcpy(m_items[i].pszTitle, items[i].pszTitle);
		m_items[i].pszPageLabel	= new TCHAR[lstrlen(items[i].pszPageLabel) + 1];
		lstrcpy((LPTSTR) m_items[i].pszPageLabel, items[i].pszPageLabel);
		m_items[i].img			= new CxImage;
		m_items[i].img->LoadResource(FindResource(items[i].hInstance, MAKEINTRESOURCE(items[i].imgID), TEXT("PNG")), CXIMAGE_FORMAT_PNG, items[i].hInstance);
		if(!m_items[i].img->IsValid())
		{
			delete m_items[i].img;
			m_items[i].img = NULL;
		} else
		{
			if((int) m_items[i].img->GetWidth() > m_iconSize.cx)
			{
				m_iconSize.cx = m_items[i].img->GetWidth();
			}
			if((int) m_items[i].img->GetHeight() > m_iconSize.cy)
			{
				m_iconSize.cy = m_items[i].img->GetHeight();
			}
		}
		SIZE szText;
		GetTextExtentPoint32(dc, m_items[i].pszTitle, lstrlen(m_items[i].pszTitle), &szText);
		if(szText.cx > m_textSize.cx)
		{
			m_textSize.cx = szText.cx;
		}
		if(szText.cy > m_textSize.cy)
		{
			m_textSize.cy = szText.cy;
		}
	}
	m_count = count;
	SelectObject(dc, oldfnt);
	ReleaseDC(m_hWnd, dc);

	m_itemSize.cy = max(m_textSize.cy, m_iconSize.cy) + 4;
	m_itemSize.cx = m_textSize.cx + m_iconSize.cx + 8;

	RECT rcClient;
	GetClientRect(m_hWnd, &rcClient);
	if(rcClient.right - rcClient.left > m_itemSize.cx)
	{
		m_itemSize.cx = rcClient.right - rcClient.left;
	}

	RecalcLayout();
}

void CTreeTabCtl::RecalcLayout(void)
{
	int y = 0;
	for(int i=0; i < m_count; i++)
	{
		m_items[i].rcItem.top		= y;
		m_items[i].rcItem.left		= 0;
		m_items[i].rcItem.bottom	= m_items[i].rcItem.top + m_itemSize.cy;
		m_items[i].rcItem.right		= m_items[i].rcItem.left + m_itemSize.cx;
		y += m_itemSize.cy;
	}
	InvalidateRect(m_hWnd, NULL, FALSE);
}

void CTreeTabCtl::DrawItem(HDC hdc, _TTAB_PAGE* item, LPRECT rcItem)
{
	if(item->state & TTAB_STATE_ACTIVE)
	{
		FillRect(hdc, rcItem, (HBRUSH) (COLOR_HIGHLIGHT + 1));
		SetTextColor(hdc, GetSysColor(COLOR_HIGHLIGHTTEXT));
	} else
	{
		FillRect(hdc, rcItem, (HBRUSH) (COLOR_WINDOW + 1));
		SetTextColor(hdc, GetSysColor(COLOR_WINDOWTEXT));
	}
	if(item->img)
	{
		int x = rcItem->left + 2 + m_iconSize.cx / 2 - item->img->GetWidth() / 2;
		int y = rcItem->top + (rcItem->bottom - rcItem->top) / 2 - item->img->GetHeight() / 2;
		item->img->Draw(hdc, x, y);
	}
	RECT rcText = *rcItem;
	rcText.left += m_iconSize.cx + 6;
	SetBkMode(hdc, TRANSPARENT);
	DrawText(hdc, item->pszTitle, -1, &rcText, DT_LEFT | DT_VCENTER | DT_SINGLELINE);
	if((item->state & TTAB_STATE_ACTIVE) && GetFocus() == m_hWnd)
	{
		DrawFocusRect(hdc, rcItem);
	}
}

void CTreeTabCtl::ActivateItem(int idx)
{
	if(m_items[idx].state & TTAB_STATE_ACTIVE)
	{
		return;
	}
	for(int i=0; i < m_count; i++)
	{
		if(i != idx)
		{
			m_items[i].state &= ~TTAB_STATE_ACTIVE;
		}
	}
	m_items[idx].state |= TTAB_STATE_ACTIVE;
	ShowPage(&m_items[idx]);
	for(int i=0; i < m_count; i++)
	{
		if(i != idx && m_items[i].hWndPage)
		{
			ShowWindow(m_items[i].hWndPage, SW_HIDE);
		}
	}
	InvalidateRect(m_hWnd, NULL, FALSE);
}

void CTreeTabCtl::ShowPage(_TTAB_PAGE* item)
{
	if(!item->hWndPage)
	{
		item->hWndPage = CreateDialogIndirectParam(item->hInstance, item->pResource, GetParent(m_hWnd), (DLGPROC) item->pfnDlgProc, item->lParam);
	}
	if(item->hWndPage)
	{
		SetWindowPos(item->hWndPage, m_hWnd, m_pagesRect.left, m_pagesRect.top,
			m_pagesRect.right - m_pagesRect.left,
			m_pagesRect.bottom - m_pagesRect.top, SWP_NOACTIVATE);
		ShowWindow(item->hWndPage, SW_SHOWNA);
	}
	if(m_pagesLabel && item->pszPageLabel)
	{
		SetWindowText(m_pagesLabel, item->pszPageLabel);
	}
	SendMessage(GetParent(m_hWnd), WM_COMMAND, MAKEWPARAM(GetDlgCtrlID(m_hWnd), TTABN_CHANGED), (LPARAM) m_hWnd);
}

CTabLabel::CTabLabel()
{
}

CTabLabel::~CTabLabel()
{
	if(m_hFont) DeleteObject(m_hFont);
}

LRESULT CALLBACK CTabLabel::WndProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam)
{
	CTabLabel* pThis = NULL;
	if(IsWindow(hWnd))
	{
		pThis = (CTabLabel*)GetProp(hWnd, TEXT("CTabLabel_this"));
	}

	if(pThis || uMessage == WM_CREATE)
	{
		switch (uMessage)
		{
		case WM_SETTEXT:
			{
				LRESULT res = DefWindowProc(hWnd, uMessage, wParam, lParam);
				InvalidateRect(pThis->m_hWnd, NULL, TRUE);
				return res;
			}
			break;
		case WM_CREATE:
			{
				LPCREATESTRUCT lpcs = (LPCREATESTRUCT)lParam;
				pThis = new CTabLabel;
				SetProp(hWnd, TEXT("CTabLabel_this"), (HANDLE) pThis);
				pThis->m_hWnd = hWnd;
			}
			break;
		case WM_DESTROY:
			delete pThis;
			RemoveProp(hWnd, TEXT("CTabLabel_this"));
			break;
		case WM_PAINT:
			{
				PAINTSTRUCT ps;
				HDC hdc = BeginPaint(pThis->m_hWnd, &ps);
				HFONT oldfnt = (HFONT) SelectObject(ps.hdc, pThis->m_hFont);
				pThis->OnPaint(hdc);
				SelectObject(ps.hdc, oldfnt);
				EndPaint(pThis->m_hWnd, &ps);
			}
			return 0;
		}
	}

	return DefWindowProc(hWnd, uMessage, wParam, lParam);
}

void CTabLabel::OnPaint(HDC hdc)
{
	TCHAR	m_text[500];
	GetWindowText(m_hWnd, m_text, 500);
	RECT rcClient;
	GetClientRect(m_hWnd, &rcClient);
	rcClient.left += 4;
	SetTextColor(hdc, GetSysColor(COLOR_HIGHLIGHTTEXT));
	SetBkColor(hdc, GetSysColor(COLOR_HIGHLIGHT));
	SetBkMode(hdc, OPAQUE);
	DrawText(hdc, m_text, -1, &rcClient, DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_NOPREFIX);
}

void CTabLabel::OnCreate()
{
	RECT rcClient;
	GetClientRect(m_hWnd, &rcClient);
	NONCLIENTMETRICS ncm;
 	ZeroMemory(&ncm, sizeof(ncm));
 	ncm.cbSize = sizeof(ncm);
 	SystemParametersInfo(SPI_GETNONCLIENTMETRICS, ncm.cbSize, &ncm, 0);
	ncm.lfMessageFont.lfHeight = rcClient.bottom - rcClient.top;
	ncm.lfMessageFont.lfWidth = 0;
	ncm.lfMessageFont.lfWeight = FW_BOLD;
 	m_hFont = CreateFontIndirect(&ncm.lfMessageFont);
}

void CTreeTabCtl::SetParams(TTAB_OPTIONS* params)
{
	m_pagesRect = params->rcPage;
	m_pagesLabel = params->hWndPageLabel;
}

void CTreeTabCtl::Apply(void)
{
	for(int i=0; i < m_count; i++)
	{
		if(m_items[i].hWndPage)
		{
			NMHDR	hdr;
			hdr.code		= PSN_APPLY;
			hdr.hwndFrom	= GetParent(m_hWnd);
			hdr.idFrom		= 0;
			SendMessage(m_items[i].hWndPage, WM_NOTIFY, 0, (LPARAM) &hdr);
		}
	}
}

void CTreeTabCtl::OnKeyDown(UINT vk)
{
	switch(vk)
	{
	case VK_UP:
		for(int i=0; i < m_count; i++)
		{
			if(m_items[i].state & TTAB_STATE_ACTIVE)
			{
				if(i > 0)
				{
					ActivateItem(i - 1);
					SetFocus(m_hWnd);
				}
				break;
			}
		}
		break;
	case VK_DOWN:
		for(int i=0; i < m_count; i++)
		{
			if(m_items[i].state & TTAB_STATE_ACTIVE)
			{
				if(i + 1 < m_count)
				{
					ActivateItem(i + 1);
					SetFocus(m_hWnd);
				}
				break;
			}
		}
		break;
	}
}

