#include "stdafx.h"
#include <windowsx.h>
#include "weblinkctl.h"
#include <shellapi.h>

void RegisterWebLinkClass(HINSTANCE hInst)
{
	WNDCLASS wc;
	if(!GetClassInfo(hInst, WEBLINK_CLASS, &wc))
	{
		ZeroMemory(&wc, sizeof(wc));
		wc.style          = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
		wc.lpfnWndProc    = (WNDPROC)CWebLinkCtl::WndProc;
		wc.cbClsExtra     = 0;
		wc.cbWndExtra     = 0;
		wc.hInstance      = hInst;
		wc.hIcon          = NULL;
		wc.hCursor        = LoadCursor(NULL, IDC_HAND);
		wc.hbrBackground  = (HBRUSH)(COLOR_BTNFACE + 1);
		wc.lpszMenuName   = NULL;
		wc.lpszClassName  = WEBLINK_CLASS;

		RegisterClass(&wc);
	}
}


CWebLinkCtl::CWebLinkCtl(void)
{
	m_url = NULL;
	m_hWnd = NULL;
	m_TextFont = NULL;
	m_OverFont = NULL;
	m_trackLeave = FALSE;
}

CWebLinkCtl::~CWebLinkCtl(void)
{
	FREE_CLEAR_STR(m_url);
	if(m_TextFont) DeleteObject(m_TextFont);
	if(m_OverFont) DeleteObject(m_OverFont);
}

LRESULT CALLBACK CWebLinkCtl::WndProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam)
{
	CWebLinkCtl* pThis = NULL;
	if(IsWindow(hWnd))
	{
		pThis = (CWebLinkCtl*)GetProp(hWnd, TEXT("CWebLinkCtl_this"));
	}

	if(pThis || uMessage == WM_CREATE)
	{
		switch (uMessage)
		{
		case WM_GETDLGCODE:
			return DLGC_STATIC;
		case WM_LBUTTONDOWN:
			pThis->OnLButtonDown(GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));
			return 0;
		case WM_MOUSEMOVE:
			pThis->OnMouseMove(GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));
			return 0;
		case WM_MOUSELEAVE:
			pThis->OnMouseLeave();
			return 0;
		case WM_CREATE:
			{
				LPCREATESTRUCT lpcs = (LPCREATESTRUCT)lParam;
				pThis = new CWebLinkCtl;
				SetProp(hWnd, TEXT("CWebLinkCtl_this"), (HANDLE) pThis);
				pThis->m_hWnd = hWnd;
				pThis->OnCreate();
			}
			break;
		case WM_DESTROY:
			delete pThis;
			RemoveProp(hWnd, TEXT("CWebLinkCtl_this"));
			break;
		case WM_PAINT:
			{
				PAINTSTRUCT ps;
				HDC hdc = BeginPaint(pThis->m_hWnd, &ps);
				pThis->OnPaint(hdc);
				EndPaint(pThis->m_hWnd, &ps);
			}
			return 0;
		case WM_SETURL:
			MAKE_STR(pThis->m_url, ((LPTSTR) lParam));
			InvalidateRect(pThis->m_hWnd, NULL, TRUE);
			return 0;
		}
	}

	return DefWindowProc(hWnd, uMessage, wParam, lParam);
}

void CWebLinkCtl::OnCreate(void)
{
	NONCLIENTMETRICS ncm;
 	ZeroMemory(&ncm, sizeof(ncm));
 	ncm.cbSize = sizeof(ncm);
 	SystemParametersInfo(SPI_GETNONCLIENTMETRICS, ncm.cbSize, &ncm, 0);
	m_TextFont = CreateFontIndirect(&ncm.lfMessageFont);
	ncm.lfMessageFont.lfUnderline = TRUE;
	m_OverFont = CreateFontIndirect(&ncm.lfMessageFont);
}

void CWebLinkCtl::OnLButtonDown(int x, int y)
{
	ShellExecute(m_hWnd, TEXT("open"), m_url, NULL, NULL, SW_SHOWNORMAL);
}

void CWebLinkCtl::OnPaint(HDC hdc)
{
	RECT rcClient;
	GetClientRect(m_hWnd, &rcClient);
	HFONT oldfnt = NULL;
	if(!m_trackLeave)
	{
		oldfnt = (HFONT) SelectObject(hdc, m_TextFont);
	} else
	{
		oldfnt = (HFONT) SelectObject(hdc, m_OverFont);
	}

	TCHAR txt[500];
	txt[0] = 0;
	GetWindowText(m_hWnd, txt, 255);

	SetBkMode(hdc, TRANSPARENT);
	SetTextColor(hdc, RGB(0, 0, 0xFF));
	DrawText(hdc, txt, -1, &rcClient, DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_NOPREFIX);

	SelectObject(hdc, oldfnt);
}

void CWebLinkCtl::OnMouseMove(int x, int y)
{
	if(!m_trackLeave)
	{
		TrackMouseLeave(m_hWnd, FALSE);
		m_trackLeave = TRUE;
		InvalidateRect(m_hWnd, NULL, TRUE);
	}
}

void CWebLinkCtl::OnMouseLeave(void)
{
	TrackMouseLeave(m_hWnd, TRUE);
	m_trackLeave = FALSE;
	InvalidateRect(m_hWnd, NULL, TRUE);
}
