// spacerbtn.h: interface for the CSpacerBtn class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_MEMMONBTN_H__3C592E7C_85D8_4277_BC0D_A56DFD76A318__INCLUDED_)
#define AFX_MEMMONBTN_H__3C592E7C_85D8_4277_BC0D_A56DFD76A318__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "ximage.h"
#include <shldisp.h>
#include "skin.h"
#include "rwlock.h"

class CCmdHistory : public IEnumString,
					public CTArray<LPSTR>
{
protected:
	DWORD	m_ObjRefCount;
	int		m_enumIDX;
	CRWLock m_sync;
public:
	CCmdHistory();
	~CCmdHistory();

	void ReadLock() { m_sync.ReadLock(); }
	void ReadUnlock() { m_sync.ReadUnlock(); }

	void WriteLock() { m_sync.WriteLock(); }
	void WriteUnlock() { m_sync.WriteUnlock(); }

	//IUnknown methods
	STDMETHODIMP QueryInterface(REFIID, LPVOID*);
	STDMETHODIMP_(DWORD) AddRef();
	STDMETHODIMP_(DWORD) Release();

	//IEnumString
	STDMETHODIMP Next(ULONG celt, LPOLESTR * rgelt, ULONG * pceltFetched);
	STDMETHODIMP Reset(void);
	STDMETHODIMP Skip(ULONG celt);
	STDMETHODIMP Clone(IEnumString ** ppenum);
};

#define CMDLINE_CLASS	"TLB_CMDLINE_WINDOW"

struct PROPS_SETTINGS
{
	UINT	tab;

};

class CCmdlineBtn : public ITlbButton
{
	DWORD			m_version;
	RECT			m_rc;
	DWORD			m_ID;
	ITlbContainer*	m_container;

	HWND			m_hWnd;
	char*			m_text;
	UINT			m_buffSize;
	UINT			m_strSize;
	UINT			m_insertPos;
	UINT			m_left;
	HFONT			m_font;
	BOOL			m_createdByHotKey;
	WNDPROC			m_oldProc;
	UINT			m_historySize;
	char			m_skinPath[MAX_PATH];
	BOOL			m_autoHide;
	int*			m_hotKeys;
	int				m_hkCount;

	CTArray<ALIAS>	m_aliases;
	CTArray<SEARCH_ENGINE>	m_engines;
	CCmdHistory*	m_history;

	IAutoComplete*	m_autoComplete;

    DWORD			sz;
	UINT			m_align;
	WORD			m_hotKey;
	BOOL			m_enableASList;
	BOOL			m_enableAUTOAPPEND;
	//alias list dialog
	int				m_a_sortType;
	int				m_a_sortCol;
	int				m_a_colWidth[3];
	int				m_a_colOrder[3];
	//search engines list dialog
	int				m_s_sortType;
	int				m_s_sortCol;
	int				m_s_colWidth[2];
	int				m_s_colOrder[2];

	int				m_curTab;

	char			m_fontName[32];
	BOOL			m_fontBold;
	BOOL			m_fontItalic;
    int				m_fontSize;

	CSkin			m_skin;
protected:
	DWORD m_ObjRefCount;
public:
	CCmdlineBtn();
	virtual ~CCmdlineBtn();

	HINSTANCE m_hResModule;

	static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam);

	//IUnknown methods
	STDMETHODIMP QueryInterface(REFIID, LPVOID*);
	STDMETHODIMP_(DWORD) AddRef();
	STDMETHODIMP_(DWORD) Release();

	//ITlbButton methods
    STDMETHODIMP Load(IStream *data);
    STDMETHODIMP Save(IStream *data);
    STDMETHODIMP SetRect(RECT *rcItem);
    STDMETHODIMP SetContainer(ITlbContainer *cont);
    STDMETHODIMP SetID(int btnID);
    STDMETHODIMP Draw(HANDLE hDC, RECT *rcItem);
    STDMETHODIMP OnMouseOver(long keys, int x, int y);
    STDMETHODIMP OnMouseLeave(long keys);
    STDMETHODIMP OnMouseEnter(long keys, int x, int y);
	STDMETHODIMP Create(IStream* data);
	STDMETHODIMP StateChanged(int state, int* needRedraw);
	STDMETHODIMP GetDescription(char* szDescription);
	STDMETHODIMP OnLButtonDown(long key, int x, int y);
	STDMETHODIMP OnLButtonUp(long key, int x, int y);
	STDMETHODIMP Run();
	STDMETHODIMP DoAction(int action);
	STDMETHODIMP SetName(LPCSTR szName);
	STDMETHODIMP_(int) QueryContextMenu(HMENU hMenu, int index, int cmdFirst, int cmdLast);
	STDMETHODIMP InvokeCommand(int ID);
	STDMETHODIMP SetLCID(DWORD dwLCID, HMODULE hResModule);
	STDMETHODIMP GetIconLocation(LPSTR szLocation);
	STDMETHODIMP SetIconLocation(LPCSTR szLocation);
	STDMETHODIMP GetButtonInfo(BUTTONINFO* btninfo);
	STDMETHODIMP Invoke(DWORD id, WPARAM wParam, LPARAM lParam);

private:
	void Create(HWND parent);
	LRESULT OnKeyDown(UINT key);
	void InitHotKeys(void);
	void CpyAliases(CTArray<ALIAS>& dst, CTArray<ALIAS>& src);
	void CpyEngines(CTArray<SEARCH_ENGINE>& dst, CTArray<SEARCH_ENGINE>& src);
public:
	void GetEditRect(LPRECT rcItem, LPRECT rcEdit);
private:
	void ClearHotkeys(void);
protected:
	void Execute(char* cmd, char* workdir);
private:
	void SetAutoCompleteFlags(void);
	void FixHistorySize(void);
	void StopHotkeys(void);
public:
	char* ParseAlias(char* cmdLine, char* txt);
private:
	void CreateCmdFont(void);
	void SetTextFont(void);
	void CreateAutocomplete(void);
};

#endif // !defined(AFX_MEMMONBTN_H__3C592E7C_85D8_4277_BC0D_A56DFD76A318__INCLUDED_)
