// MemMonBtn.cpp: implementation of the CSpacerBtn class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "spacerbtn.h"
#include "propsdlg.h"
#include <stdio.h>
#include <shobjidl.h>
#include <shlwapi.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSpacerBtn::CSpacerBtn()
{
	sz = 20;

	m_imgFile[0] = 0;
	m_img = 0;
	m_align = 0;

	m_version = 1;
	m_ObjRefCount = 1;
	m_container = NULL;

	g_DllRefCount++;
}

CSpacerBtn::~CSpacerBtn()
{
	if(m_img)
	{
		delete m_img;
		m_img = NULL;
	}
	g_DllRefCount--;
}


//////////////////////////////////////////////////////////////////////
// IUnknown methods
//////////////////////////////////////////////////////////////////////

STDMETHODIMP CSpacerBtn::QueryInterface(REFIID riid, LPVOID *ppReturn)
{
	*ppReturn = NULL;

	//IUnknown
	if(IsEqualIID(riid, IID_ITlbButton))
	   {
	   *ppReturn = (ITlbButton*) this;
	   }

	if(IsEqualIID(riid, IID_IUnknown))
	   {
	   *ppReturn = (IUnknown*) this;
	   }

	if(*ppReturn)
	   {
	   (*(LPUNKNOWN*)ppReturn)->AddRef();
	   return S_OK;
	   }

	return E_NOINTERFACE;
}

STDMETHODIMP_(DWORD) CSpacerBtn::AddRef()
{
	return ++m_ObjRefCount;
}

STDMETHODIMP_(DWORD) CSpacerBtn::Release()
{
	if(--m_ObjRefCount == 0)
	{
		delete this;
		return 0;
	}
	return m_ObjRefCount;
}


//////////////////////////////////////////////////////////////////////
// ITlbButton methods
//////////////////////////////////////////////////////////////////////


// Load parameters for button
STDMETHODIMP CSpacerBtn::Load(IStream *data)
{
	DWORD loadedVersion = 0;
	ULONG cbRead = 0;
	data->Read(&loadedVersion, sizeof(DWORD), &cbRead);
	data->Read(&sz, sizeof(sz), &cbRead);
	if(loadedVersion >= 1)
	{
		data->Read(m_imgFile, MAX_PATH, &cbRead);
		data->Read(&m_align, sizeof(m_align), &cbRead);
	}
	LoadImage();
	return S_OK;
}

// Save button parameters
STDMETHODIMP CSpacerBtn::Save(IStream *data)
{
	ULONG cbWritten;
	data->Write(&m_version, sizeof(DWORD), &cbWritten);
	data->Write(&sz, sizeof(sz), &cbWritten);
	data->Write(m_imgFile, MAX_PATH, &cbWritten);
	data->Write(&m_align, sizeof(m_align), &cbWritten);
	return S_OK;
}

// Sets button rectangle
STDMETHODIMP CSpacerBtn::SetRect(RECT *rcItem)
{
	m_rc = rcItem[0];
	return S_OK;
}



// Set button container
// NOTE: do not call AddRef to store container pointer
STDMETHODIMP CSpacerBtn::SetContainer(ITlbContainer *cont)
{
	m_container = cont;
	return S_OK;
}

STDMETHODIMP CSpacerBtn::SetID(int btnID)
{
	m_ID = btnID;
	return S_OK;
}

STDMETHODIMP CSpacerBtn::Draw(HANDLE hDC, RECT *rcItem)
{
	if(!m_img) return S_OK;
	CONTAINER_PARAMS params;
	params.cbSize = sizeof(CONTAINER_PARAMS);
	m_container->GetParams(&params);
	int x = rcItem->left;
	int y = rcItem->top;
	if(params.style & CTS_ISMENU)
	{
		switch(m_align)
		{
		case 0:
			x = rcItem->left + (rcItem->right - rcItem->left) / 2 - m_img->GetWidth() / 2;
			break;
		case 1:
			x = rcItem->left;
			break;
		case 2:
			x = rcItem->right - m_img->GetWidth();
			break;
		}
	} else
	{
		if(params.dwViewMode & DBIF_VIEWMODE_VERTICAL)
		{
			switch(m_align)
			{
			case 0:
				x = rcItem->left + (rcItem->right - rcItem->left) / 2 - m_img->GetWidth() / 2;
				break;
			case 1:
				x = rcItem->left;
				break;
			case 2:
				x = rcItem->right - m_img->GetWidth();
				break;
			}
		} else
		{
			switch(m_align)
			{
			case 0:
				y = rcItem->top + (rcItem->bottom - rcItem->top) / 2 - m_img->GetHeight() / 2;
				break;
			case 1:
				y = rcItem->top;
				break;
			case 2:
				y = rcItem->bottom - m_img->GetHeight();
				break;
			}
		}
	}
	m_img->Draw((HDC) hDC, x, y);
	return S_OK;
}

STDMETHODIMP CSpacerBtn::OnMouseOver(long keys, int x, int y)
{
	return S_OK;
}

STDMETHODIMP CSpacerBtn::OnMouseLeave(long keys)
{
	return S_OK;
}

STDMETHODIMP CSpacerBtn::OnMouseEnter(long keys, int x, int y)
{
	return S_OK;
}

STDMETHODIMP CSpacerBtn::Create(IStream* data)
{
	if(m_container)
	{
		HWND hWndBand = m_container->GetBandWND();
		CPropsDlg dlg;
		dlg.sz = sz;
		dlg.m_align = m_align;
		strcpy(dlg.m_imgFile, m_imgFile);
		if(dlg.DoModal(hWndBand) == IDOK)
		{
			sz = dlg.sz;
			m_align = dlg.m_align;
			strcpy(m_imgFile, dlg.m_imgFile);
			return Save(data);
		}
	}
	return S_FALSE;
}


STDMETHODIMP CSpacerBtn::StateChanged(int state, int* needRedraw)
{
	return S_OK;
}

STDMETHODIMP CSpacerBtn::GetDescription(char* szDescription)
{
	return S_FALSE;
}

STDMETHODIMP CSpacerBtn::OnLButtonDown(long key, int x, int y)
{
	return S_FALSE;
}

STDMETHODIMP CSpacerBtn::OnLButtonUp(long key, int x, int y)
{
	return S_FALSE;
}

STDMETHODIMP CSpacerBtn::Run()
{
	return S_OK;
}



STDMETHODIMP CSpacerBtn::DoAction(int action)
{
	if(action == CTMS_PROPERTIES)
	{
		if(m_container)
		{
			HWND hWndBand = m_container->GetBandWND();
			CPropsDlg dlg;
			dlg.sz = sz;
			dlg.m_align = m_align;
			strcpy(dlg.m_imgFile, m_imgFile);
			if(dlg.DoModal(hWndBand) == IDOK)
			{
				sz = dlg.sz;
				m_align = dlg.m_align;
				strcpy(m_imgFile, dlg.m_imgFile);
				LoadImage();
				if(m_container)
				{
					m_container->UpdateButtonInfo(UPDATE_REDRAW | UPDATE_SIZE);
				}
				return m_container->SaveButton();
			}
		}
	}
	return S_OK;
}

STDMETHODIMP_(int) CSpacerBtn::QueryContextMenu(HMENU hMenu, int index, int cmdFirst, int cmdLast)
{
	return 0;
}

STDMETHODIMP CSpacerBtn::InvokeCommand(int ID)
{
	return S_OK;
}


STDMETHODIMP CSpacerBtn::SetLCID(DWORD dwLCID, HMODULE hResModule)
{
	return S_OK;
}

STDMETHODIMP CSpacerBtn::GetIconLocation(LPSTR szLocation)
{
	return S_FALSE;
}

STDMETHODIMP CSpacerBtn::SetIconLocation(LPCSTR szLocation)
{
	return S_FALSE;
}

STDMETHODIMP CSpacerBtn::SetName(LPCSTR szName)
{
	return S_OK;
}

STDMETHODIMP CSpacerBtn::GetButtonInfo(BUTTONINFO* btninfo)
{
	if(btninfo->dwMask & BTNINFO_MASK_ACTIONS)
	{
		btninfo->dwActions = CTMS_PROPERTIES;
	}
	if(btninfo->dwMask & BTNINFO_MASK_MODEFLAG)
	{
		btninfo->dwModeFlags = BTN_FLAG_PLANE | BTN_FLAG_NOTIPS;
	}
	if(btninfo->dwMask & BTNINFO_MASK_SIZE ||
	   btninfo->dwMask & BTNINFO_MASK_ACTUALSIZE)
	{
		if(m_container)
		{
			CONTAINER_PARAMS params;
			params.cbSize = sizeof(CONTAINER_PARAMS);
			m_container->GetParams(&params);
			if((params.style & CTS_ISMENU) && !(btninfo->dwMask & BTNINFO_MASK_ACTUALSIZE))
			{
				if(m_img)
				{
					btninfo->szBtnSize.cy = m_img->GetHeight();
					btninfo->szBtnSize.cx = m_img->GetWidth();
				} else
				{
					btninfo->szBtnSize.cy = sz;
				}
			} else
			{
				if(params.dwViewMode & DBIF_VIEWMODE_VERTICAL)
				{
					if(m_img)
					{
						btninfo->szBtnSize.cy = m_img->GetHeight();
					} else
					{
						btninfo->szBtnSize.cy = sz;
					}
				} else
				{
					if(m_img)
					{
						btninfo->szBtnSize.cx = m_img->GetWidth();
					} else
					{
						btninfo->szBtnSize.cx = sz;
					}
				}
			}
		}

		//int n = m_showSeconds ? 6 : 4;
		//int nn = m_showSeconds ? 2 : 1;
		//btninfo->szBtnSize.cx = (m_digWidth + 1) * n + 6 + (m_dotWidth + 1) * nn;
	}
	if(btninfo->dwMask & BTNINFO_MASK_DESCRIPTION)
	{
		char* szDescription = (char*) btninfo->strData;
		szDescription[0] = 0;
	}
	return S_OK;
}

STDMETHODIMP CSpacerBtn::Invoke(DWORD id, WPARAM wParam, LPARAM lParam)
{
	return S_FALSE;
}


void CSpacerBtn::LoadImage(void)
{
	if(m_img)
	{
		delete m_img;
		m_img = NULL;
	}
	if(PathFileExists(m_imgFile))
	{
		m_img = new CxImage;
		m_img->Load(m_imgFile, CXIMAGE_FORMAT_PNG);
		if(!m_img->IsValid())
		{
			delete m_img;
			m_img = NULL;
		}
	}
}
