#include "tlbpdkcore.h"
#include ".\plugin.h"
#include "tlbpdklib.h"

extern HINSTANCE   g_hResInst;
extern HINSTANCE   g_hInst;

CPlugin::CPlugin(CTlbButton* btn)
{
	m_ObjRefCount = 1;
	if(!btn)
	{
		m_btn = CreatePlugin();
	} else
	{
		m_btn = btn;
	}
	g_DllRefCount++;
}

CPlugin::~CPlugin(void)
{
	delete m_btn;
	g_DllRefCount--;
}

//////////////////////////////////////////////////////////////////////
// IUnknown methods
//////////////////////////////////////////////////////////////////////

STDMETHODIMP CPlugin::QueryInterface(REFIID riid, LPVOID *ppReturn)
{
	*ppReturn = NULL;

	//IUnknown
	if(IsEqualIID(riid, IID_ITlbButton))
	{
		*ppReturn = (ITlbButton*) this;
	}
	if(IsEqualIID(riid, IID_IUnknown))
	{
		*ppReturn = (IUnknown*) ((ITlbButton*) this);
	}

	if(*ppReturn)
	{
		(*(LPUNKNOWN*)ppReturn)->AddRef();
		return S_OK;
	}

	return E_NOINTERFACE;
}

STDMETHODIMP_(DWORD) CPlugin::AddRef()
{
	return ++m_ObjRefCount;
}

STDMETHODIMP_(DWORD) CPlugin::Release()
{
	if(--m_ObjRefCount == 0)
	{
		delete this;
		return 0;
	}
	return m_ObjRefCount;
}

//////////////////////////////////////////////////////////////////////
// ITlbButton methods
//////////////////////////////////////////////////////////////////////

// Load parameters for button
STDMETHODIMP CPlugin::Load(IStream *data)
{
	if(m_btn->Load(data))
	{
		return S_OK;
	}
	return S_FALSE;
}

// Save button parameters
STDMETHODIMP CPlugin::Save(IStream *data)
{
	if(m_btn->Save(data))
	{
		return S_OK;
	}
	return S_FALSE;
}

// Sets button rectangle
STDMETHODIMP CPlugin::SetRect(RECT *rcItem)
{
	m_btn->SetRect(rcItem);
	return S_OK;
}

// Set button container
// NOTE: do not call AddRef to store container pointer
STDMETHODIMP CPlugin::SetContainer(ITlbContainer *cont)
{
	m_container = cont;
	m_btn->SetContainer(this);
	return S_OK;
}

STDMETHODIMP CPlugin::SetID(int btnID)
{
	m_btn->SetButtonID(btnID);
	return S_OK;
}

STDMETHODIMP CPlugin::Draw(HANDLE hDC, RECT *rcItem)
{
	m_btn->OnDraw((HDC) hDC, rcItem);
	return S_OK;
}

STDMETHODIMP CPlugin::OnMouseOver(long keys, int x, int y)
{
	if(m_btn->OnMouseOver(keys, x, y))
	{
		return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CPlugin::OnMouseLeave(long keys)
{
	if(m_btn->OnMouseLeave(keys))
	{
		return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CPlugin::OnMouseEnter(long keys, int x, int y)
{
	if(m_btn->OnMouseEnter(keys, x, y))
	{
		return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CPlugin::Create(IStream* data)
{
	if(m_btn->OnCreate())
	{
		return Save(data);
	}
	return S_FALSE;
}


STDMETHODIMP CPlugin::StateChanged(int state, int* needRedraw)
{
	if(m_btn->OnStateChanged(state, needRedraw))
	{
		return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CPlugin::GetDescription(char* szDescription)
{
	return S_FALSE;
}

STDMETHODIMP CPlugin::OnLButtonDown(long key, int x, int y)
{
	if(m_btn->OnLButtonDown(key, x, y))
	{
		return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CPlugin::OnLButtonUp(long key, int x, int y)
{
	if(m_btn->OnLButtonUp(key, x, y))
	{
		return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CPlugin::Run()
{
	if(m_btn->OnRun())
	{
		return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CPlugin::DoAction(int action)
{
	BOOL res = FALSE;
	switch(action)
	{
	case CTMS_PROPERTIES:
		res = m_btn->OnProperties();
		break;
	case CTMS_RENAME:
		res = m_btn->OnRename();
		break;
	case CTMS_ABOUT:
		res = m_btn->OnAbout();
		break;
	case CTMS_DELETE:
		res = m_btn->OnDelete();
		break;
	}
	if(!res)
	{
		return S_FALSE;
	}
	return S_OK;
}

STDMETHODIMP_(int) CPlugin::QueryContextMenu(HMENU hMenu, int index, int cmdFirst, int cmdLast)
{
	return m_btn->QueryContextMenu(hMenu, index, cmdFirst, cmdLast);
}

STDMETHODIMP CPlugin::InvokeCommand(int ID)
{
	if(m_btn->OnContextMenuCommand(ID))
	{
		return S_OK;
	}
	return S_FALSE;
}


STDMETHODIMP CPlugin::SetLCID(DWORD dwLCID, HMODULE hResModule)
{
	m_btn->OnSetLCID(dwLCID, hResModule);
	return S_OK;
}

STDMETHODIMP CPlugin::GetIconLocation(LPSTR szLocation)
{
	m_btn->GetIconLocation((LPWSTR) szLocation);
	return S_OK;
}

STDMETHODIMP CPlugin::SetIconLocation(LPCSTR szLocation)
{
	if(m_btn->SetIconLocation((LPCWSTR) szLocation))
	{
		return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CPlugin::SetName(LPCSTR szName)
{
	m_btn->SetName((LPCWSTR) szName);
	return S_OK;
}

STDMETHODIMP CPlugin::GetButtonInfo(BUTTONINFO* btninfo)
{
	if(btninfo->dwMask & BTNINFO_MASK_ACTIONS)
	{
		btninfo->dwActions = m_btn->GetSupportedActions();
	}
	if(btninfo->dwMask & BTNINFO_MASK_MODEFLAG)
	{
		btninfo->dwModeFlags = m_btn->GetModeFlags() | BTN_FLAG_UNICODE;
	}
	if(btninfo->dwMask & BTNINFO_MASK_SIZE)
	{
		m_btn->GetSize(&btninfo->szBtnSize, FALSE);
	}
	if(btninfo->dwMask & BTNINFO_MASK_ACTUALSIZE)
	{
		m_btn->GetSize(&btninfo->szBtnSize, TRUE);
	}
	if(btninfo->dwMask & BTNINFO_MASK_TEXT)
	{
		m_btn->GetButtonText((LPWSTR) btninfo->strData);
	}
	return S_OK;
}

STDMETHODIMP CPlugin::Invoke(DWORD id, WPARAM wParam, LPARAM lParam)
{
	switch(id)
	{
	case TBTN_INVOKE_MENU_MEASUREITEM:
		m_btn->OnMeasureItem(wParam, lParam);
		return S_OK;
	case TBTN_INVOKE_MENU_DRAWITEM:
		m_btn->OnDrawItem(wParam, lParam);
		return S_OK;
	case TBTN_INVOKE_GETTIPSICONSIZE:
		{
			int* icSZ = (int*) lParam;
			*icSZ = m_btn->GetTipsIconSize(wParam);
		}
		return S_OK;
	case TBTN_INVOKE_DRAWTIPSICON:
		{
			DRAW_TIPS_ICON* dIc = (DRAW_TIPS_ICON*) lParam;
			m_btn->DrawTipsIcon(dIc->hDC, dIc->pt, wParam);
		}
		return S_OK;
	case TBTN_INVOKE_GETMENUFLAGS:
		{
			UINT* flags = (UINT*) lParam;
			*flags = m_btn->GetMenuFlags();
		}
		return S_OK;
	case TBTN_INVOKE_QUERYHOTKEY:
		{
			HKM_DATA* hkd = (HKM_DATA*) lParam;
			HOTKEY_DATA hkData;
			ZeroMemory(&hkData, sizeof(hkData));
			if(m_btn->GetHotKeyData((int) wParam, &hkData))
			{
				hkd->id = hkData.id;
				hkd->hotKey = hkData.hotKey;
				hkd->hotKeyID = hkData.hotKeyID;
				memcpy(hkd->name, hkData.name, sizeof(hkData.name));
				return S_OK;
			}
		}
		break;
	case TBTN_INVOKE_HOTKEYSCOUNT:
		{
			int* cnt = (int*) lParam;
			*cnt = m_btn->GetHotkeysCount();
		}
		return S_OK;
	case TBTN_INVOKE_ONHOTKEY:
		m_btn->OnHotkey((UINT) lParam);
		return S_OK;
	case TBTN_INVOKE_GETNAME:
		m_btn->GetButtonName((LPWSTR) lParam);
		return S_OK;
	case TBTN_INVOKE_GETTIPSICON:
		{
			HICON* lpIcon = (HICON*) lParam;
			*lpIcon = m_btn->GetTipIcon(wParam);
		}
		return S_OK;
	case TBTN_INVOKE_GETTIPSCAPTION:
		{
			LPWSTR* text = (LPWSTR*) lParam;
			*text = m_btn->GetTipCaption(wParam);
			if(!*text) return S_FALSE;
		}
		return S_OK;
	case TBTN_INVOKE_GETTOOLTIPSCOUNT:
		{
			int* tipsCount = (int*) lParam;
			*tipsCount = m_btn->GetTipsCount();
		}
		return S_OK;
	case TBTN_INVOKE_GETTOOLTIPRECT:
		{
			LPRECT rcTip = (LPRECT) lParam;
			m_btn->GetTipRect(wParam, rcTip);
		}
		return S_OK;
	case TBTN_INVOKE_GETTIPSTEXT:
		{
			LPWSTR* text = (LPWSTR*) lParam;
			*text = m_btn->GetTipText(wParam);
			if(!*text) return S_FALSE;
		}
		return S_OK;
	case TBTN_INVOKE_ONSHOW:
		m_btn->OnShow();
		return S_OK;
	case TBTN_INVOKE_ONHIDE:
		m_btn->OnHide();
		return S_OK;
	case TBTN_INVOKE_CREATEWND:
		m_btn->CreateWnd((HWND) lParam);
		return S_OK;
	case TBTN_INVOKE_ISMENU:
		{
			BOOL* im = (BOOL*) lParam;
			*im = m_btn->isMenu();
		}
		return S_OK;
	case TBTN_INVOKE_GETCHANGESHANDLE:
		{
			HANDLE* hChanges = (HANDLE*) lParam;
			*hChanges = m_btn->GetChangesHandle();
		}
		return S_OK;
	case TBTN_INVOKE_PREPARECHANGES:
		m_btn->PrepareChanges((HANDLE) lParam);
		return S_OK;
	case TBTN_INVOKE_CLOSECHANGESHANDLE:
		m_btn->CloseChangesHandle((HANDLE) lParam);
		return S_OK;
	case TBTN_INVOKE_FIRSTCHILDDATA:
		{
			ENUM_CHILDS* data = (ENUM_CHILDS*) lParam;
			CHILDS_DATA d;
			d.childData		= data->childData;
			d.enumData		= data->enumData;
			d.Name			= (LPWSTR) data->Name;
			if(m_btn->FirstChildData(&d))
			{
				data->childData		= d.childData;
				data->enumData		= d.enumData;
				data->Name			= (LPTSTR) d.Name;
				return S_OK;
			}
		}
		break;
	case TBTN_INVOKE_NEXTCHILDDATA:
		{
			ENUM_CHILDS* data = (ENUM_CHILDS*) lParam;
			CHILDS_DATA d;
			d.childData		= data->childData;
			d.enumData		= data->enumData;
			d.Name			= (LPWSTR) data->Name;
			if(m_btn->NextChildData(&d))
			{
				data->childData		= d.childData;
				data->enumData		= d.enumData;
				data->Name			= (LPTSTR) d.Name;
				return S_OK;
			} else
			{
				data->enumData		= NULL;
			}
		}
		break;
	case TBTN_INVOKE_FREECHILDDATA:
		{
			CHILDS_DATA d;
			d.childData		= (LPVOID) lParam;
			d.enumData		= NULL;
			d.Name			= NULL;
			if(m_btn->FreeChildData(&d))
			{
				return S_OK;
			}
		}
		break;
	case TBTN_INVOKE_CREATECHILD:
		{
			CREATE_CHILDS* data = (CREATE_CHILDS*) lParam;
			CREATE_CHILD_DATA d;
			d.parent		= ((CPlugin*) data->parent)->m_btn;
			d.childData		= data->childData;
			d.child			= NULL;
			if(m_btn->CreateChild(&d))
			{
				data->child = new CPlugin(d.child);
				return S_OK;
			}
		}
		break;
	case TBTN_INVOKE_READCHANGES:
		{
			UINT* changes = (UINT*) lParam;
			*changes = m_btn->ReadChanges();
		}
		return S_OK;
	case TBTN_INVOKE_GETCHILDUID:
		m_btn->GetChildUID((LPWSTR) lParam);
		return S_OK;
	case TBTN_INVOKE_COMPAREBUTTONS:
		{
			COMPARE_BUTTONS* cmpData = (COMPARE_BUTTONS*) lParam;
			COMPARE_BUTTONS_DATA cmpD;
			cmpD.btn1 = ((CPlugin*) cmpData->btn1)->m_btn;
			cmpD.btn2 = ((CPlugin*) cmpData->btn2)->m_btn;
			cmpD.fnc = cmpData->fnc;
			cmpD.result = cmpData->result;
			m_btn->CompareButtons(&cmpD);
			cmpData->result = cmpD.result;
		}
		return S_OK;
	case TBTN_INVOKE_QUERYARRANGEMENU:
		{
			QUERY_ARRANGE_MENU* amData = (QUERY_ARRANGE_MENU*) lParam;
			ARRANGE_MENU_DATA amD;
			amD.name	= (LPWSTR) amData->name;
			amD.sortID	= amData->sortID;
			if(m_btn->QueryArrangeMenu(wParam, &amD))
			{
				amData->name	= (LPTSTR) amD.name;
				amData->sortID	= amD.sortID;
				return S_OK;
			}
		}
		break;
	case TBTN_INVOKE_SUPPORTSORT:
		{
			BOOL* res = (BOOL*) lParam;
			*res = m_btn->SupportSortMenu();
		}
		return S_OK;
	}
	return S_FALSE;
}


void CPlugin::DrawItemText(HDC hDC, LPRECT rcText, LPCWSTR text)
{
	m_container->DrawItemText(hDC, rcText, (LPCSTR) text);
}

HWND CPlugin::GetToolbarWindow()
{
	return m_container->GetBandWND();
}

void CPlugin::GetIconPoint(LPPOINT ptIcon)
{
	m_container->GetIconPoint(ptIcon);
}

int	CPlugin::GetIconSize()
{
	int szIcon = 16;
	CONTAINER_PARAMS cParams = {0, 0, 0};
	cParams.cbSize = sizeof(CONTAINER_PARAMS);
	if(m_container)
	{
		m_container->GetParams(&cParams);
		szIcon = cParams.dwIconSize;
	}
	return szIcon;
}

DWORD CPlugin::GetViewMode()
{
	int viewMode = 0;
	CONTAINER_PARAMS cParams = {0, 0, 0};
	cParams.cbSize = sizeof(CONTAINER_PARAMS);
	if(m_container)
	{
		m_container->GetParams(&cParams);
		viewMode = cParams.dwViewMode;
	}
	return viewMode;
}

HFONT CPlugin::GetItemFont()
{
	HFONT hFont = 0;
	CONTAINER_PARAMS cParams = {0, 0, 0};
	cParams.cbSize = sizeof(CONTAINER_PARAMS);
	if(m_container)
	{
		m_container->GetParams(&cParams);
		hFont = (HFONT) cParams.hItemFont;
	}
	return hFont;
}

COLORREF CPlugin::GetTextColor()
{
	RECT rcMargins;
	COLORREF txtColor = 0;
	m_container->GetTextProps(&rcMargins, &txtColor);
	return txtColor;
}

void CPlugin::GetTextMargins(LPRECT rcMargins)
{
	ZeroMemory(rcMargins, sizeof(RECT));
	COLORREF txtColor = 0;
	m_container->GetTextProps(rcMargins, &txtColor);
}

void CPlugin::SetHotKey(DWORD hk, UINT hkID)
{
	m_container->Invoke(CONTAINER_CMD_SETHOTKEY, hk, hkID);
}

BOOL CPlugin::IsVisible()
{
	BOOL vis = TRUE;
	m_container->Invoke(CONTAINER_CMD_ISVISIBLE, 0, (LPARAM) &vis);
	return vis;
}

void CPlugin::Hide()
{
	m_container->Invoke(CONTAINER_CMD_HIDE, 0, 0);
}

void CPlugin::Show()
{
	m_container->Invoke(CONTAINER_CMD_SHOW, 0, 0);
}

void CPlugin::QueryCreateWindow()
{
	m_container->Invoke(CONTAINER_CMD_CREATEWNDREQUEST, 0, 0);
}

void CPlugin::OpenParentMenu()
{
	m_container->Invoke(CONTAINER_CMD_OPENWINDOW, 0, 0);
}

void CPlugin::OnKillFocus()
{
	m_container->Invoke(CONTAINER_CMD_KILLFOCUS, 0, 0);
}

void CPlugin::OnRButtonUp()
{
	m_container->Invoke(CONTAINER_CMD_RBUTTONUP, 0, 0);
}

void CPlugin::GetSavedPath(UINT objID, LPWSTR path)
{
	m_container->Invoke(CONTAINER_CMD_GETSAVEDPATH, objID, (LPARAM) path);
}

void CPlugin::SetState(UINT state)
{
	m_container->Invoke(CONTAINER_CMD_SETSTATE, state, NULL);
}

void CPlugin::SaveButton()
{
	m_container->SaveButton();
}

void CPlugin::UpdateSize()
{
	m_container->UpdateButtonInfo(UPDATE_SIZE);
}

void CPlugin::Redraw()
{
	m_container->UpdateButtonInfo(UPDATE_REDRAW);
}

void CPlugin::UpdateTips()
{
	m_container->UpdateButtonInfo(UPDATE_TOOLTIP);
}

void CPlugin::ShowMenu()
{
	m_container->Invoke(CONTAINER_CMD_SHOWMENU, 0, 0);
}

BOOL CPlugin::tboShowCtxIcons()
{
	BOOL ret = TRUE;
	m_container->Invoke(CONTAINER_CMD_GETTOOLBAROPTION, TBO_SHOWCTXICONS, (LPARAM) &ret);
	return ret;
}
